#Include-Once

Global Const $ACTION_MAXNUMBER = 2
Global Const $ACTION_FONTS = StringSplit( "Arial|Comic Sans MS|Courier|Courier New|Franklin Gothic Medium|Impact|Lucida Console|Modern MS Sans Serif|" & _
                             "MS Sans Serif|Roman|Script|Symbol|Tahoma|Times New Roman|Trebuchet|Verdana|Webdings|Wingdings|Wingdings 2|Wingdings 3" , "|")
Global Const $ACTION_TYPES_MAX = 6

#Region > Modlisation des objets "Action"

; ~~~ Prsentation de la structure ~~~

; L'objet "Action" est une reprsentation des informations pour une action customisable de l'interface du nAiO; lui sont rattachs les attributs "privs" suivants :
;  - title       [string-64]  : titre du ToolTip li au bouton customisable de l'action
;  - description [string-512] : description du ToolTip li au bouton customisable de l'action
;  - label       [string-3]   : label du bouton customisable de l'action
;  - font        [string-32]  : police du bouton customisable de l'action ( cf. [Global]$ACTION_FONTS )
;  - bold        [string-1]   : valeur gale  Y si le bouton customisable de l'action doit tre en gras
;  - size        [string-2]   : taille du texte du bouton customisable de l'action
;  - type        [string-1]   : type d'action ( entier entre 0 et $ACTION_TYPES_MAX )
;  - parameter   [string-256] : paramtre de l'action
; La liste de actions est une reprsentation des actions enregistres; lui est rattache la modlisation suivante :
;  - Array de taille n+1, o n reprsente le nombre de actions enregistres
;    - [indice 0] : valeur gale au nombre de actions enregistres, soit n
;    - [indice 1] : premire action (objet "Action")
;    - ...
;    - [indice n] : (n)ime action (objet "Action")
;  - n tant un nombre fixe $ACTION_MAXNUMBER pour cette version du nAiO, une liste vierge est reprsente par un ($ACTION_MAXNUMBER+1)-uplet [ $ACTION_MAXNUMBER , newAction() , newAction() , ... , newAction() ]



; ~~~ Constructeurs et fonctions de base ~~~

; ### newAction($title,$description,$label,$font,$bold,$size,$type,$parameter)>[Action] :
; Construit un objet "Action"  partir des donnes fournies, et retourne sa valeur
; => Tous les arguments sont optionnels
Func newAction($title="", $description="", $label="", $font="Arial", $bold="N", $size="10", $type="0", $parameter="")
	Local $action = DllStructCreate(getStructureAction())
	setActionTitle($action, $title)
	setActionDescription($action, $description)
	setActionLabel($action, $label)
	setActionFont($action, $font)
	setActionBold($action, $bold)
	setActionSize($action, $size)
	setActionType($action, $type)
	setActionParameter($action, $parameter)
	Return $action
EndFunc
; ### copyAction($oldAction)>[Action] :
; Construit un objet "Action"  partir des donnes dupliques d'un autre objet "Action", et retourne la nouvelle valeur
Func copyAction($oldAction)
	Local $newAction = newAction(), $structure = getStructureActionArray()
	For $i = 0 To getStructureActionNumber()-1
		copyActionAttribute($newAction, $oldAction, $structure[$i][1])
	Next
	Return $newAction
EndFunc
; ### deleteAction($action)>void :
; Detruit l'objet "Action" $action
Func deleteAction(ByRef $action)
	$action = 0
EndFunc
; ### isAction($action)>boolean :
; Retourne la valeur True si $action est bien un objet "Action", False sinon
Func isAction($action)
	Local $i = 0, $structure = getStructureActionArray()
	If Not IsDllStruct($action) Then Return False
	For $i=0 To getStructureActionNumber()-1
		Local $valeur = DllStructGetData($action, $structure[$i][1])
		If @error Then Return False
	Next
	Return True
EndFunc
; ### isActionDefined($action)>boolean :
; Retourne la valeur True si $action est bien un objet "Action" valide et ayant une activit, False sinon
Func isActionDefined($action)
	Select
	Case Not isAction($action)
		Return False
	Case getActionLabel($action) = ""
		Return False
	Case Int(getActionType($action)) = 0
		Return False
	EndSelect
	Return True
EndFunc
; ### toStringAction($action)>string :
; Retourne une chaine de caractres identifiant un objet "Action"
Func toStringAction($action)
	For $i = 1 To getActionTypeMax()
		If $i = Number(getActionType($action)) Then Return $i&" - "&getActionTypeName($i)
	Next
	Return "0 - "&getActionTypeName(0)
EndFunc
; ### newListeActions()>array :
; Cre une liste vierge d'objets "Action"
Func newListeActions()
	Local $retour[$ACTION_MAXNUMBER+1]
	$retour[0] = $ACTION_MAXNUMBER
	For $i = 1 To $ACTION_MAXNUMBER
		$retour[$i] = newAction()
	Next
	Return $retour
EndFunc
; ### deleteListeActions($liste)>void :
; Ecrase $liste avec une liste vierge d'objets "Action" aprs avoir dtruit chaque objet "Action" prsent
Func deleteListeActions(ByRef $liste)
	If IsArray($liste) Then
		For $i = 0 To UBound($liste)-1
			$liste[$i] = 0
		Next
	EndIf
	$liste = newListeActions()
EndFunc



; ~~~ Fonctions de dbugage ~~~

; ### displayAction($action)>void :
; Affiche les informations de $action dans une fentre type MsgBox
Func displayActions($liste)
	For $i = 1 To getListeActionsLength($liste)
		displayAction(getActionInListAt($liste,$i),$i&"/"&getListeActionsLength($liste))
	Next
EndFunc
; ### displayActions($liste)>void :
; Afficher les informations des lments de $liste dans des fentres ordonnes type MsgBox
Func displayAction($action, $title="")
	If isAction($action) Then
		Local $structure = getStructureActionArray(), $msg = _
			"Action.toString: '" & toStringAction($action) & "'" & @CRLF & _
			"Struct Size: " & DllStructGetSize($action) & @CRLF & _
			"Struct pointer: " & DllStructGetPtr($action) & @CRLF & _
			"Data:"
		For $i = 0 To getStructureActionNumber()-1
			$msg &= @CRLF & @TAB & $structure[$i][1]&": '"&getActionAttribute($action, $structure[$i][1])&"'"
		Next
		MsgBox(0, "Action"&cond($title="",""," ["&$title&"]"), "{"&@CRLF&$msg&@CRLF&"}" )
	Else
		MsgBox(16, "Action"&cond($title="",""," ["&$title&"]"), "!!! Cette action n'est pas un objet conforme !!!" )
	EndIf
EndFunc



; ~~~ Fonctions Get/Set pour les attributs index, alias et avatar ~~~

; ### getActionTitle($action)>string :
; Lit l'attribut title de $action, et retourne sa valeur
Func getActionTitle($action)
	Return getActionAttribute($action, "title")
EndFunc
; ### setActionTitle($action,$title)>void :
; Ecrit la valeur $title dans l'attribut title de $action
Func setActionTitle(ByRef $action, $title)
	setActionAttribute($action, "title", $title)
EndFunc
; ### getActionDescription($action)>string :
; Lit l'attribut description de $action, et retourne sa valeur
Func getActionDescription($action)
	Return StringReplace(getActionAttribute($action, "description"),"\n",@CRLF)
EndFunc
; ### setActionDescription($action,$description)>void :
; Ecrit la valeur $description dans l'attribut description de $action
Func setActionDescription(ByRef $action, $description)
	setActionAttribute($action, "description", StringReplace($description,@CRLF,"\n"))
EndFunc
; ### getActionLabel($action)>string :
; Lit l'attribut label de $action, et retourne sa valeur
Func getActionLabel($action)
	Return getActionAttribute($action, "label")
EndFunc
; ### setActionLabel($action,$label)>void :
; Ecrit la valeur $label dans l'attribut label de $action
Func setActionLabel(ByRef $action, $label)
	setActionAttribute($action, "label", $label)
EndFunc
; ### getActionFont($action)>string :
; Lit l'attribut font de $action, et retourne sa valeur
Func getActionFont($action)
	Return getActionAttribute($action, "font")
EndFunc
; ### setActionFont($action,$font)>void :
; Ecrit la valeur $font dans l'attribut font de $action
Func setActionFont(ByRef $action, $font)
	setActionAttribute($action, "font", $font)
EndFunc
; ### getActionBold($action)>string :
; Lit l'attribut bold de $action, et retourne sa valeur
Func getActionBold($action)
	Return getActionAttribute($action, "bold")
EndFunc
; ### setActionBold($action,$bold)>void :
; Ecrit la valeur $bold dans l'attribut bold de $action
Func setActionBold(ByRef $action, $bold)
	setActionAttribute($action, "bold", $bold)
EndFunc
; ### isActionBold($action)>string :
; Renvoie True si l'attribut bold de $action est gal  la valeur "Y", False sinon
Func isActionBold($action)
	Return getActionBold($action) = "Y"
EndFunc
; ### getActionSize($action)>string :
; Lit l'attribut size de $action, et retourne sa valeur
Func getActionSize($action)
	Return getActionAttribute($action, "size")
EndFunc
; ### setActionSize($action,$size)>void :
; Ecrit la valeur $size dans l'attribut size de $action
Func setActionSize(ByRef $action, $size)
	setActionAttribute($action, "size", $size)
EndFunc
; ### getActionType($action)>string :
; Lit l'attribut type de $action, et retourne sa valeur
Func getActionType($action)
	Return getActionAttribute($action, "type")
EndFunc
; ### setActionType($action,$type)>void :
; Ecrit la valeur $type dans l'attribut type de $action
Func setActionType(ByRef $action, $type)
	setActionAttribute($action, "type", $type)
EndFunc
; ### getActionTypeName($type)>string :
; Retourne une chaine de caractres identifiant un type $type d'action (compris entre 0 et getActionTypeMax())
Func getActionTypeName($type)
	Local $typeName = getInternationalisationText($nAiO_languageResources, "MAction_type"&$type&"_label")
	If $typeName = "" Then Return getInternationalisationText($nAiO_languageResources, "MAction_type0_label")
	Return $typeName
EndFunc
; ### getActionTypeNax()>int :
; Retourne le nombre total de diffrents types d'actions possible (sans compter le type inactif)
Func getActionTypeMax()
	Return $ACTION_TYPES_MAX
EndFunc
; ### getActionParameter($action)>string :
; Lit l'attribut parameter de $action, et retourne sa valeur
Func getActionParameter($action)
	Return getActionAttribute($action, "parameter")
EndFunc
; ### setActionParameter($action,$parameter)>void :
; Ecrit la valeur $parameter dans l'attribut parameter de $action
Func setActionParameter(ByRef $action, $parameter)
	setActionAttribute($action, "parameter", $parameter)
EndFunc



; ~~~ Fonctions de listing de actions ~~~

; ### getActionInListAt($liste,$i)>[Action] :
; Retourne l'objet "Action" prsent  l'indice $i dans la liste de actions $liste
; => Si l'indice $i ne contient pas un objet "Action", la chaine vide "" est retourne
Func getActionInListAt($liste,$i)
	If isAction($liste[$i]) Then
		Return $liste[$i]
	Else
		Return ""
	EndIf
EndFunc
; ### setActionInListAt($liste,$i,$action)>void :
; Ecrit l'objet "Action" $action  l'indice $i de la liste de actions $liste
; => Si l'indice $i est invalide, aucune action n'est effectue
Func setActionInListAt(ByRef $liste,$i,$action)
	If $i>0 And $i<=getListeActionsLength($liste) Then
		$liste[$i] = $action
	EndIf
EndFunc
; ### resetActionInListAt($liste,$i)>void :
; Rinitialise l'objet "Action" $action  l'indice $i
; => Si l'indice $i est invalide, aucune action n'est effectue
Func resetActionInListAt(ByRef $liste,$i)
	setActionInListAt($liste,$i,newAction())
EndFunc
; ### getListeActionsLength($liste)>int :
; Retourne le nombre d'objets "Action" prsent dans la liste de actions $liste
Func getListeActionsLength($liste)
	Return $liste[0]
EndFunc
; ### setListeActionsLength($liste,$length)>void :
; Redimensionne  $length la taille de la liste de actions $liste
Func setListeActionsLength(ByRef $liste, $length)
	If $length>=0 Then
		$liste[0] = $length
		ReDim $liste[getListeActionsLength($liste)+1]
	EndIf
EndFunc



; ~~~ Fonctions pour mashall/unmarshall ~~~

; ### convertLigneToAction($ligne)>[Compte] :
; Transforme la chaine de caractres $ligne en objet "Action" (unmarshall)
Func convertLigneToAction($ligne)
	Local $param = StringSplit($ligne,"")
	If $param[0] = getStructureActionNumber() Then Return newAction($param[1], $param[2], $param[3], $param[4], $param[5], $param[6], $param[7], $param[8])
	SetError(1)
EndFunc
; ### convertActionToLigne($action)>string :
; Srialise l'objet "Action" $action en chaine de caractres (marshall)
Func convertActionToLigne($action)
	Return getActionTitle($action)&""&StringReplace(getActionDescription($action),@CRLF,"\n")&""&getActionLabel($action)&""&getActionFont($action)&""&getActionBold($action)&""&getActionSize($action)&""&getActionType($action)&""&getActionParameter($action)
EndFunc
; ### convertFichierToListeActions($chemin,$liste)>void :
; Lit les donnes du fichier de config $chemin, et enregistre la liste de actions correspondante dans $liste
; => Les donnes de $liste sont crases
Func convertFichierToListeActions($chemin, ByRef $liste)
	$liste = newListeActions()
	For $i = 1 To $ACTION_MAXNUMBER
		Local $ligne = IniRead($chemin, "Custom","Action"&$i,"")
		If $ligne <> "" Then setActionInListAt($liste,$i,convertLigneToAction($ligne))
	Next
EndFunc
; ### convertListeActionsToFichier($liste,$chemin)>void :
; Lit les donnes de la liste d'actions $liste et enregistre les donnes correspondantes dans le fichier de config $chemin
; => Les donnes du fichier $chemin sont crases
Func convertListeActionsToFichier($liste, $chemin)
	For $i = 1 To $ACTION_MAXNUMBER
		Local $action = getActionInListAt($liste,$i)
		IniWrite($chemin, "Custom", "Action"&$i, convertActionToLigne($action))
	Next
EndFunc



; ~~~ Fonctions internes  la modlisation,  ne pas utiliser hors de ce fichier ~~~

; ### getStructureActionArray()>matrix :
; Retourne une matrice contenant les attributs et leurs formats, pour structurer un objet "Action"
Func getStructureActionArray()
	Local $structure[8][3] = [["char","title",64], ["char","description",512], ["char","label",3], ["char","font",32], ["char","bold",1], ["char","size",2], ["char","type",1], ["char","parameter",256]]
	Return $structure
EndFunc
; ### getStructureActionNumber()>int :
; Retourne le nombre d'attributs ncessaires pour structurer un objet "Action"
Func getStructureActionNumber()
	Return UBound(getStructureActionArray(),1)
EndFunc
; ### getStructureAction()>string :
; Retourne une chaine de caractres contenant les attributs et leurs formats, pour structurer un objet "Action"
Func getStructureAction()
	Local $i = 0, $string = "", $structure = getStructureActionArray()
	For $i=0 To getStructureActionNumber()-1
		If $string <> "" Then $string &= ";"
		$string &= $structure[$i][0] & " " & $structure[$i][1] & cond($structure[$i][2]<>"", "["&$structure[$i][2]&"]", "")
	Next
	Return $string
EndFunc
; ### getActionAttribute($action,$attribut)>string :
; Retourne l'attribut $attribut de l'objet "Action" $action
Func getActionAttribute($action, $attribut)
	Return DllStructGetData($action, $attribut)
EndFunc
; ### setActionAttribute($action,$attribut,$valeur)>void :
; Ecrit la valeur $valeur dans l'attribut $attribut de l'objet "Action" $action
Func setActionAttribute(ByRef $action, $attribut, $valeur)
	DllStructSetData($action, $attribut, StringReplace(StringReplace(StringReplace($valeur,"","#"),@CR,""),@LF,""))
EndFunc
; ### copyActionAttribute($newAction,$oldAction,$attribut)>void :
; Ecrit la valeur de l'attribut $attribut de l'objet "Action" $oldAction dans celui de l'objet "Action" $newAction
Func copyActionAttribute(ByRef $newAction, $oldAction, $attribut)
	setActionAttribute($newAction, $attribut, getActionAttribute($oldAction, $attribut))
EndFunc

#EndRegion

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
