#Include-Once

Global $test[1] = [0]

#Region > Modlisation des objets "Alerte"

; ~~~ Prsentation de la structure ~~~

; L'objet "Alerte" est une reprsentation d'une alerte, priodique ou non, et apparaissant sous diverses formes; lui sont rattachs les attributs "privs" suivants :
;  - index       [entier]      : indice permettant un accs rapide  l'alerte. les valeurs de "index" doivent tre diffrents deux  deux pour chaque alerte existante
;  - dateA       [chaine-16]   : date de dbut de l'alerte (format : 'YYYY/MM/JJ HH:MM')
;  - dateB       [chaine-16]   : date de fin de l'alerte   (format : 'YYYY/MM/JJ HH:MM')
;  - periodic    [chaine-16]   : priodicit de l'alerte   (format : '^([1-9]\d* ((jour|heure|minute)s?|mois))?$')
;  - title       [chaine-64]   : titre du message  afficher sur l'alerte
;  - descript    [chaine-256]  : description de l'alerte, avec les sauts de lignes remplacs par \n
;  - occurrences [chaine-1024] : liste des occurrences de l'alerte (dates au format 'YYYY/MM/JJ HH:MM' spares par des ',')
; La liste d'alertes est une reprsentation des alertes enregistres; lui est rattache la modlisation suivante :
;  - Array de taille n+1, o n reprsente le nombre d'alertes enregistres
;    - [indice 0] : valeur gale au nombre d'alertes enregistres, soit n
;    - [indice 1] : premire alerte (objet "Alerte")
;    - ...
;    - [indice n] : (n)ime alerte (objet "Alerte")
;  - Une liste vierge d'alertes est donc reprsente ainsi : Dim $array[1] = [0]



; ~~~ Constructeurs et fonctions de base ~~~

; ### newAlerte($index,$dateA,$dateB,$periodic,$title,$descript)>[Alerte] :
; Construit un objet "Alerte"  partir des donnes $index, $dateA, $dateB, $periodic, $title et $descript, et retourne sa valeur
; => Tous les paramtres sont tous optionnels
Func newAlerte($index="", $dateA="", $dateB="", $periodic="", $title="", $descript="")
	Local $alerte = DllStructCreate(getStructureAlerte())
	If $index <> "" Then setAlerteIndex($alerte, $index)
	setAlerteDateA($alerte, $dateA)
	setAlerteDateB($alerte, $dateB)
	setAlertePeriodic($alerte, $periodic)
	setAlerteTitle($alerte, $title)
	setAlerteDescript($alerte, $descript)
	setAlerteOccurrences($alerte, "")
	If $dateB = "" Then
		setAlerteDateB($alerte, $dateA)
	ElseIf _DateDiff("s",$dateA,$dateB) < 0 Then
		setAlerteDateB($alerte, $dateA)
	EndIf
	Return $alerte
EndFunc
; ### copyAlerte($oldAlerte)>[Alerte] :
; Construit un objet "Alerte"  partir des donnes dupliques d'un autre objet "Alerte", et retourne la nouvelle valeur
Func copyAlerte($oldAlerte)
	Local $newAlerte = newAlerte(), $structure = getStructureAlerteArray()
	For $i = 0 To getStructureAlerteNumber()-1
		copyAlerteAttribute($newAlerte, $oldAlerte, $structure[$i][1])
	Next
	Return $newAlerte
EndFunc
; ### deleteAlerte($alerte)>void :
; Detruit l'objet "Alerte" $alerte
Func deleteAlerte(ByRef $alerte)
	$alerte = 0
EndFunc
; ### isAlerte($alerte)>boolean :
; Retourne la valeur True si $alerte est bien un objet "Alerte", False sinon
Func isAlerte($alerte)
	Local $i = 0, $valeur = "", $structure = getStructureAlerteArray()
	If Not IsDllStruct($alerte) Then
		Return False
	EndIf
	For $i=0 To UBound($structure)-1
		Local $valeur = DllStructGetData($alerte, $structure[$i][1])
		If @error Then Return False
	Next
	Return True
EndFunc
; ### toStringAlerte($alerte)>string :
; Retourne une chaine de caractres identifiant un objet "Alerte"
Func toStringAlerte($alerte)
	Return getAlerteIndex($alerte)&" - "&getAlerteTitle($alerte)
EndFunc
; ### newListeAlertes()>array :
; Cre une liste vierge d'alertes
Func newListeAlertes() ; => tableau
	Local $retour[1]
	setListeAlertesLength($retour,0)
	Return $retour
EndFunc
; ### deleteListeAlertes($liste)>void :
; Ecrase $liste avec une liste vierge d'objets "Alerte" aprs avoir dtruit chaque objet "Alerte" prsent
Func deleteListeAlertes(ByRef $liste)
	If IsArray($liste) Then
		For $i = 0 To UBound($liste)-1
			$liste[$i] = 0
		Next
	EndIf
	$liste = newListeAlertes()
EndFunc



; ~~~ Fonctions de dbugage ~~~

; ### displayAlerte($alerte)>void :
; Affiche les informations de $alerte dans une fentre type MsgBox
Func displayAlertes($liste)
	For $i = 1 To _Min(UBound($liste)-1,$liste[0])
		displayAlerte($liste[$i],$i&"/"&$liste[0]&cond($liste[0]=UBound($liste)-1,""," (TAILLE:"&UBound($liste)-1&")"))
	Next
EndFunc
; ### displayAlertes($liste)>void :
; Afficher les informations des lments de $liste dans des fentres ordonnes type MsgBox
Func displayAlerte($alerte, $title="")
	Local $fullTitle = "Alerte"
	If $title<>"" Then $fullTitle = $fullTitle & " ["&$title&"]"
	If isAlerte($alerte) Then
		Local $structure = getStructureAlerteArray(), $msg = _
			"Struct Size: " & DllStructGetSize($alerte) & @CRLF & _
			"Struct pointer: " & DllStructGetPtr($alerte) & @CRLF & _
			"Data:"
		For $i = 0 To getStructureAlerteNumber()-1
			$msg &= @CRLF & @TAB & $structure[$i][1]&": '"&getAlerteAttribute($alerte, $structure[$i][1])&"'"
		Next
		MsgBox(0, $fullTitle, "{"&@CRLF&$msg&@CRLF&"}" )
	Else
		MsgBox(16, $fullTitle, "!!! Cette alerte n'est pas un objet conforme !!!" )
	EndIf
EndFunc



; ~~~ Fonctions Get/Set pour les attributs index, alias et avatar ~~~

; ### getAlerteIndex($alerte)>string :
; Lit l'attribut index de $alerte
Func getAlerteIndex($alerte)
	Return getAlerteAttribute($alerte, "index")
EndFunc
; ### setAlerteIndex($alerte,$index)>void :
; Ecrit la valeur $index dans l'attribut index de $alerte
Func setAlerteIndex(ByRef $alerte, $index)
	setAlerteAttribute($alerte, "index", $index)
EndFunc
; ### getAlerteDateA($alerte)>string :
; Lit l'attribut dateA de $alerte
Func getAlerteDateA($alerte)
	Return getAlerteAttribute($alerte, "dateA")
EndFunc
; ### setAlerteDateA($alerte,$dateA)>void :
; Ecrit la valeur $dateA dans l'attribut dateA de $alerte
Func setAlerteDateA(ByRef $alerte, $dateA)
	setAlerteAttribute($alerte, "dateA", $dateA)
EndFunc
; ### getAlerteDateB($alerte)>string :
; Lit l'attribut dateB de $alerte
Func getAlerteDateB($alerte)
	Return getAlerteAttribute($alerte, "dateB")
EndFunc
; ### setAlerteDateB($alerte,$dateB)>void :
; Ecrit la valeur $dateB dans l'attribut dateB de $alerte
Func setAlerteDateB(ByRef $alerte, $dateB)
	setAlerteAttribute($alerte, "dateB", $dateB)
EndFunc
; ### getAlertePeriodic($alerte)>string :
; Lit l'attribut periodic de $alerte
Func getAlertePeriodic($alerte)
	Return getAlerteAttribute($alerte, "periodic")
EndFunc
; ### setAlertePeriodic($alerte,$periodic)>void :
; Ecrit la valeur $periodic dans l'attribut periodic de $alerte
Func setAlertePeriodic(ByRef $alerte, $periodic)
	setAlerteAttribute($alerte, "periodic", $periodic)
EndFunc
; ### getAlerteTitle($alerte)>string :
; Lit l'attribut title de $alerte
Func getAlerteTitle($alerte)
	Return getAlerteAttribute($alerte, "title")
EndFunc
; ### setAlerteTitle($alerte,$title)>void :
; Ecrit la valeur $title dans l'attribut title de $alerte
Func setAlerteTitle(ByRef $alerte, $title)
	setAlerteAttribute($alerte, "title", $title)
EndFunc
; ### getAlerteDescript($alerte)>string :
; Lit l'attribut descript de $alerte et remplace les '\n' prsents par des sauts de ligne
Func getAlerteDescript($alerte)
	Return StringReplace(getAlerteAttribute($alerte, "descript"),"\n",@CRLF)
EndFunc
; ### setAlerteDescript($alerte,$descript)>void :
; Ecrit la valeur $descript dans l'attribut descript de $alerte en remplaant les sauts de ligne prsent par des '\n'
Func setAlerteDescript(ByRef $alerte, $descript)
	setAlerteAttribute($alerte, "descript", StringReplace($descript,@CRLF,"\n"))
EndFunc
; ### getAlerteOccurrences($alerte)>array :
; Lit l'attribut occurrences de $alerte et le renvoye sous la forme d'un tableau
Func getAlerteOccurrences($alerte)
	Local $vide[1] = [0], $chaine = getAlerteAttribute($alerte, "occurrences")
	If $chaine = "" Then Return $vide
	Return StringSplit($chaine,",")
EndFunc
; ### setAlerteOccurrences($alerte,$ocurrences)>void :
; Ecrit la valeur $occurrences dans l'attribut occurrences de $alerte
; => Si la valeur $occurrences est au format array, elle est transforme en string au format dtermin plus haut (dates au format 'YYYY/MM/JJ HH:MM' spares par des ',')
Func setAlerteOccurrences(ByRef $alerte, $occurrences)
	If IsString($occurrences) Then
		setAlerteAttribute($alerte, "occurrences", $occurrences)
	ElseIf IsArray($occurrences) Then
		Local $chaine = ""
		For $i = 1 to _Min(20,$occurrences[0])
			$chaine = $chaine & "," & $occurrences[$i]
		Next
		$chaine = StringTrimLeft($chaine,1)
		setAlerteOccurrences($alerte, $chaine)
	EndIf
EndFunc
; ### getAlertePeriodicNumber($alerte)>int :
; Lit l'attribut periodic de $alerte et retourne le nombre de minutes correspondantes
Func getAlertePeriodicNumber($alerte)
	Local $periodic = getAlertePeriodic($alerte)
	Local $valeurStr = StringSplit($periodic," "), $valeur = 0
	If $valeurStr[0] = 2 Then
		$valeur = Number($valeurStr[1]) * _
			cond(StringRegExp($valeurStr[2],"^ans?$"),    60*24*31*12, _
			cond(StringRegExp($valeurStr[2],"^mois$"),    60*24*31, _
			cond(StringRegExp($valeurStr[2],"^jours?$"),  60*24, _
			cond(StringRegExp($valeurStr[2],"^heures?$"), 60, _
			cond(StringRegExp($valeurStr[2],"^minutes?$"), 1, _
			0)))))
	EndIf
	Return $valeur
EndFunc



; ~~~ Fonctions de listing d'alertes ~~~

; ### addToListeAlertes($liste,$alerte,$force)>void :
; Ajoute l'objet "Alerte" $alerte  la fin de la liste d'alertes $liste
; => La valeur  l'indice 0 est incrmente automatiquement si un objet "Alerte" ayant la mme valeur index existe dj dans la liste
; => Avec $force gal  False, l'objet $alerte n'est pas ajout  la liste si un objet "Alerte" ayant la mme valeur index y existe dj
; => Le parametre $force est optionnel
Func addToListeAlertes(ByRef $liste, $alerte, $force = True)
	Local $indexAlerte = getAlerteIndex($alerte)
	If isAlerte(getAlerteInListFromIndex($liste,$indexAlerte)) Then
		If $force Then
			setAlerteIndex($alerte, $indexAlerte+1)
			addToListeAlertes($liste, $alerte)
		EndIf
	Else
		setListeAlertesLength($liste,getListeAlertesLength($liste)+1)
		setAlerteInListAt($liste,getListeAlertesLength($liste),$alerte)
	EndIf
EndFunc
; ### delInListeAlertesAt($liste,$i)>[Alerte] :
; Efface l'objet "Alerte" prsent  l'indice $i de la liste d'alertes $liste, et retourne l'objet "Alerte" qui a t retir
; => Si l'indice $i n'est pas sens contenir un objet "Alerte" ($i<1 ou $i>n), la chaine vide "" est retourne
Func delInListeAlertesAt(ByRef $liste, $i)
	Local $deleted = ""
	If $i>=1 And $i<=getListeAlertesLength($liste) Then
		$deleted = getAlerteInListAt($liste,$i)
		For $j = $i To getListeAlertesLength($liste)-1
			setAlerteInListAt($liste,$j,getAlerteInListAt($liste,$j+1))
		Next
		setListeAlertesLength($liste,getListeAlertesLength($liste)-1)
	EndIf
	Return $deleted
EndFunc
; ### delInListeAlertesFromToString($liste,$toStringAlerte)>[Alerte] :
; Efface l'objet "Alerte" ayant $toStringAlerte comme identifiant, de la liste d'alertes $liste, et retourne l'objet "Alerte" qui a t retir
; => Si aucun objet "Alerte" ne correspond au critre, la chaine vide "" est retourne
Func delInListeAlertesFromToString(ByRef $liste, $toStringAlerte)
	Local $i = 1, $deleted = ""
	While $i <= $liste[0]
		If toStringAlerte($liste[$i]) = $toStringAlerte Then ExitLoop
		$i = $i + 1
	WEnd
	If $i <= $liste[0] Then
		$deleted = $liste[$i]
		For $j = $i To $liste[0]-1
			$liste[$j] = $liste[$j+1]
		Next
		$liste[0] = $liste[0]-1
		ReDim $liste[$liste[0]+1]
	EndIf
	Return $deleted
EndFunc
; ### moveInListeAlertesFromToString($liste, $toStringAlerte1, $toStringAlerte2)>void :
; Retire l'objet "Alerte" ayant $toStringAlerte1 comme identifiant, de la liste d'alertes $liste, et dplace les objets "Alerte"
; dans la liste afin que l'objet "Alerte" retir puisse tre  la place de celui ayant $toStringAlerte2 comme identifiant
; => Si aucun objet "Alerte" ne correspond au critre $toStringAlerte1 ou $toStringAlerte2, aucune action n'est effectue
Func moveInListeAlertesFromToString(ByRef $liste, $toStringAlerte1, $toStringAlerte2)
	For $i = 1 To getListeAlertesLength($liste)
		If toStringAlerte(getAlerteInListAt($liste,$i)) = $toStringAlerte1 Then ExitLoop
	Next
	For $j = 1 To getListeAlertesLength($liste)
		If toStringAlerte(getAlerteInListAt($liste,$j)) = $toStringAlerte2 Then ExitLoop
	Next
	If $i <> $j And $i <= getListeAlertesLength($liste) And $j <= getListeAlertesLength($liste) Then
		Local $alerteTemp = getAlerteInListAt($liste,$i), $step = -1
		If $i<$j Then $step = 1
		For $k = $i To $j-$step Step $step
			setAlerteInListAt($liste,$k,getAlerteInListAt($liste,$k+$step))
		Next
		setAlerteInListAt($liste,$j,$alerteTemp)
	EndIf
EndFunc
; ### getAlerteInListFromIndex($liste,$index)>[Alerte] :
; Retourne l'objet "Alerte" prsent dans la liste d'alertes $liste ayant son attribut index gal  $index
; => Si aucun objet "Alerte" ne rpond au critre, la chaine vide "" est retourne
Func getAlerteInListFromIndex($liste,$index)
	For $i = 0 To UBound($liste)-1
		If isAlerte($liste[$i]) And getAlerteIndex($liste[$i])=$index Then
			Return $liste[$i]
		EndIf
	Next
	Return "" ; Si rien trouv
EndFunc
; ### getAlerteInListFromToString($liste,$toString)>[Alerte] :
; Retourner l'objet "Alerte" prsent dans la liste d'alertes $liste ayant $toStringAlerte comme identifiant
; => Si aucun objet "Alerte" ne rpond au critre, la chaine vide "" est retourne
Func getAlerteInListFromToString($liste,$toString)
	For $i = 0 To UBound($liste)-1
		If isAlerte($liste[$i]) And toStringAlerte($liste[$i])=$toString Then
			Return $liste[$i]
		EndIf
	Next
	Return "" ; Si rien trouv
EndFunc
; ### setAlerteInListFromToString($liste,$toStringAlerte,$dateA,$dateB,$periodic,$title,$descript)>[Alerte] :
; Modifie les informations de l'objet "Alerte" prsent dans la liste d'alertes $liste ayant $toStringAlerte comme identifiant
; par les valeurs $dateA,$dateB,$periodic,$title et $descript, et retourne la valeur de l'objet "Alerte" avant modification
; => Si aucun objet "Alerte" ne rpond au critre, la chaine vide "" est retourne
Func setAlerteInListFromToString(ByRef $liste, $toStringAlerte, $dateA,$dateB,$periodic,$title,$descript)
	Local $modified = ""
	For $i = 0 To UBound($liste)-1
		If isAlerte($liste[$i]) And toStringAlerte($liste[$i])=$toStringAlerte Then
			$modified = copyAlerte($liste[$i])
			setAlerteDateA($liste[$i],$dateA)
			setAlerteDateB($liste[$i],$dateB)
			setAlertePeriodic($liste[$i],$periodic)
			setAlerteTitle($liste[$i],$title)
			setAlerteDescript($liste[$i],$descript)
		EndIf
	Next
	Return $modified
EndFunc
; ### getAlerteInListAt($liste,$i)>[Alerte] :
; Retourne l'objet "Alerte" prsent  l'indice $i dans la liste d'alertes $liste
; => Si l'indice $i ne contient pas un objet "Alerte", la chaine vide "" est retourne
Func getAlerteInListAt($liste,$i)
	If isAlerte($liste[$i]) Then
		Return $liste[$i]
	Else
		Return ""
	EndIf
EndFunc
; ### setAlerteInListAt($liste,$i,$alerte)>void :
; Ecrit l'objet "Alerte" $alerte  l'indice $i de la liste d'alertes $liste
; => Si l'indice $i est invalide, aucune action n'est effectue
Func setAlerteInListAt(ByRef $liste,$i,$alerte)
	If $i>0 And $i<=getListeAlertesLength($liste) Then
		$liste[$i] = $alerte
	EndIf
EndFunc
; ### getListeAlertesLength($liste)>int :
; Retourne le nombre d'objets "Alerte" prsent dans la liste d'alertes $liste
Func getListeAlertesLength($liste)
	Return $liste[0]
EndFunc
; ### setListeAlertesLength($liste,$length)>void :
; Redimensionne  $length la taille de la liste d'alertes $liste
Func setListeAlertesLength(ByRef $liste, $length)
	If $length>=0 Then
		$liste[0] = $length
		ReDim $liste[getListeAlertesLength($liste)+1]
	EndIf
EndFunc
; ### occursAlerteEntreDates($alerte,$avant,$apres)>array :
; Retourne la liste des dates aux quelles l'objet "Alerte" $alerte s'est dclench entre les dates $avant et $apres, le premier lment de la liste tant le nombre de dclenchements
Func occursAlerteEntreDates($alerte,$avant,$apres)
	Local $resultat[1] = [0]
	If Not isAlerte($alerte) Then Return $resultat
	Local $periodic = getAlertePeriodicNumber($alerte)*60
	Local $avI = changeDateFormat($avant), $apI = changeDateFormat($apres)
	Local $deb = changeDateFormat(getAlerteDateA($alerte)), $fin = changeDateFormat(getAlerteDateB($alerte))
	If $avI = "" Or $apI = "" Or $deb = "" Or $fin = "" Then Return $resultat
	Local $stamp = "s", $periodicToUse = $periodic, $delay = 0
	If $periodic >= 60*60*24*31 And $periodic < 60*60*24*31*12 Then
		$stamp = "M"
		$periodicToUse = Round($periodic/(60*60*24*31),0)
		$delay = 1
	ElseIf $periodic >= 60*60*24*31*12 Then
		$stamp = "Y"
		$periodicToUse = Round($periodic/(60*60*24*31*12),0)
		$delay = 1
	EndIf
	If $periodicToUse = 0 Then
		$periodicToUse = 1
		$fin = $deb
		$delay = 0
	EndIf
	Local $delta_avant_deb = _DateDiff($stamp, $deb, $avI)/$periodicToUse, _
		  $delta_apres_deb = _DateDiff($stamp, $deb, $apI)/$periodicToUse, _
		  $delta_fin_deb   = _DateDiff($stamp, $deb, $fin)/$periodicToUse
	Local $intervalle[2] = [Ceiling(_Max(0,$delta_avant_deb))+$delay,Floor(_Min($delta_apres_deb,$delta_fin_deb))]
	$resultat[0] = _Max($intervalle[1]-$intervalle[0]+1,0)
	ReDim $resultat[$resultat[0]+1]
	For $i = 1 To $resultat[0]
		$resultat[$i] = _DateAdd($stamp,($intervalle[0]+($i-1))*$periodicToUse,$deb)
	Next
	Return $resultat
EndFunc
; ### getNextAlertesInListAt($liste,$now)>array[Alerte] :
; Retourne la liste des alertes parmi la liste $liste, dont le dclenchement sera la date la plus proche dans le futur, aprs la date $now
Func getNextAlertesInListAt($liste, $now)
	Local $vide[1] = [0], $date = ""
	Local $resultat = $vide, $extended = 0
	For $i = 1 To getListeAlertesLength($liste)
		Local $alerte = getAlerteInListAt($liste, $i)
		Local $nextDates = occursAlerteEntreDates($alerte,$now,getAlerteDateB($alerte))
		; Si l'alerte n'a plus d'occurence future, on l'ignore
		For $j = 1 To $nextDates[0]
			Local $diffFromDate = _DateDiff("s", $nextDates[$j], $date), $diffFromNow = _DateDiff("s", $now, $nextDates[$j])
			If $diffFromNow < 0 Then
				; La date est avant le moment actuel $now, on l'ignore
			ElseIf $date = "" Or $diffFromDate > 0 Then
				; Aucune date n'a pour le moment t trouve, ou bien l'alerte trouve est plus rcente que celles choisies pour le moment
				Redim $resultat[2]
				$resultat[0] = 1
				$resultat[1] = $alerte
				$date = $nextDates[$j]
				$extended = $diffFromNow
			ElseIf $diffFromDate = 0 Then
				; L'alerte trouve est au mme moment que celles choisies pour le moment
				$resultat[0] = $resultat[0]+1
				Redim $resultat[$resultat[0]+1]
				$resultat[$resultat[0]] = $alerte
			EndIf
		Next
	Next
	SetExtended($extended)
	Return $resultat
EndFunc



; ~~~ Fonctions pour l'affichage des dclencheurs ~~~

; ### getAlerteTriggerAsDate($date,$error,$format)>string :
; Transforme la date $date en une chaine de caractres correspondant au format de date $format
; => Si la date est invalide, la valeur $error est retourne
Func getAlerteTriggerAsDate($date, $error="[Date invalide]", $format="{C} {J}/{M}/{A}  {h}:{m}:{s}")
	If Not IsDeclared("NAIO_Dates") Then Local $NAIO_Dates = newList()
	Local $dateArray, $timeArray, $retour = $format, $AMPM = "PM"
	Local $split = StringSplit($date,"/ :")
	If @error Or $split[0] <> 6 Then Return $error
	Local $jourDeLaSemaine = _DateToDayOfWeekISO(Number($split[1]), Number($split[2]), Number($split[3]))
	If @error Or $jourDeLaSemaine = 0 Or $jourDeLaSemaine > $NAIO_Dates[0] Then Return $error
	; Date
	If StringInStr($retour, "{C}") Then
		$retour = StringReplace($retour, "{C}", $NAIO_Dates[$jourDeLaSemaine], 0, 1)
	EndIf
	$retour = StringReplace($retour, "{A}", $split[1], 0, 1)
	$retour = StringReplace($retour, "{M}", $split[2], 0, 1)
	$retour = StringReplace($retour, "{J}", $split[3], 0, 1)
	; Time
	If StringInStr($retour, "{t}") Then
		Switch Number($split[4])
		Case 0
			$split[4] = "12"
		Case 1 To 12
			$AMPM = "AM"
		Case 13 To 23
			$split[4] = Number($split[4])-12
			$split[4] = cond($split[4]>9,"","0")&$split[4]
		Case Else
			Return $error
		EndSwitch
		$retour = StringReplace($retour, "{t}", $AMPM, 0, 1)
	EndIf
	$retour = StringReplace($retour, "{h}", $split[4], 0, 1)
	$retour = StringReplace($retour, "{m}", $split[5], 0, 1)
	$retour = StringReplace($retour, "{s}", $split[6], 0, 1)
	Return $retour
EndFunc
; ### getAlerteTriggerAsDelay($delay,$error,$separator)>string :
; Transforme la duree $delay en une chaine de caractres correspondant au format de dure spcifi par les traductions et par $separator
; => Si la dure est invalide, la valeur $error est retourne
Func getAlerteTriggerAsDelay($delay, $error="[Dure invalide]",$separator=", ")
	If Not IsDeclared("NAIO_Delays") Then Local $NAIO_Delays = newArray(2)
	Local $now = _NowCalc(), $types = StringSplit("YMwDhns", ""), $valeur=$delay
	Local $next = _DateAdd("s",$delay,$now)
	Local $resultat = $types
	For $i = 1 To $resultat[0]
		Local $decalage = _DateDiff($types[$i], $now, $next)
		If @error Then Return $error
		$resultat[$i] = $decalage
		$now = _DateAdd($types[$i], $decalage, $now)
		If @error Then Return $error
	Next
	Local $retourArray = $resultat, $n = 0
	For $i = 1 To $retourArray[0]
		If $resultat[$i] = 0 Then
			$retourArray[$i] = ""
		Else
			$n = $n + 1
			$retourArray[$i] = $resultat[$i] & " " & $NAIO_Delays[$i][cond($resultat[$i]>1,1,0)]
		EndIf
	Next
	If $n = 0 Then Return $error
	Local $retourArrayCleaned[$n+1] = [$n], $j = 1
	For $i = 1 To $retourArray[0]
		If $retourArray[$i] <> "" Then
			$retourArrayCleaned[$j] = $retourArray[$i]
			$j = $j + 1
		EndIf
	Next
	Local $retour = ""
	For $i = 1 To $retourArrayCleaned[0]
		$retour = $retour & cond($retour="","",$separator) & $retourArrayCleaned[$i]
	Next
	Return $retour
EndFunc



; ~~~ Fonctions pour mashall/unmarshall ~~~

; ### convertLigneToAlerte($ligne)>[Alerte] :
; Transforme la chaine de caractres $ligne en objet "Alerte" (unmarshall)
Func convertLigneToAlerte($ligne)
	Local $param = StringSplit($ligne,"")
	If $param[0] <= getStructureAlerteNumber() Then Return newAlerte($param[1],$param[2],$param[3],$param[4],$param[5],$param[6])
	SetError(1)
EndFunc
; ### convertAlerteToLigne($alerte)>string :
; Srialise l'objet "Alerte" $alerte en chaine de caractres (marshall)
Func convertAlerteToLigne($alerte)
	Return getAlerteIndex($alerte)&""&getAlerteDateA($alerte)&""&getAlerteDateB($alerte)&""&getAlertePeriodic($alerte)&""&getAlerteTitle($alerte)&""&StringReplace(getAlerteDescript($alerte),@CRLF,"\n")
EndFunc
; ### convertFichierToListeAlertes($chemin,$liste)>void :
; Lit les donnes du fichier de config $chemin, et enregistre la liste d'alertes correspondante dans $liste
; => Les donnes de $liste sont crases
Func convertFichierToListeAlertes($chemin, ByRef $liste, $pass="")
	$liste = newListeAlertes()
	Local $fichier = FileOpen($chemin, 0)
	While $fichier <> -1
		Local $ligne = FileReadLine($fichier)
		If @error <> 0 Then ExitLoop
		Local $alerte = convertLigneToAlerte($ligne)
		If @error = 0 Then addToListeAlertes($liste, $alerte)
	WEnd
	FileClose($fichier)
EndFunc
; ### convertListeAlertesToFichier($liste,$chemin)>void :
; Lit les donnes de la liste d'alertes $liste et enregistre les donnes correspondantes dans le fichier de config $chemin
; => Les donnes du fichier $chemin sont crases
Func convertListeAlertesToFichier($liste, $chemin, $pass="")
	Local $texte = ""
	For $i = 1 To $liste[0]
		If $texte <> "" Then $texte = $texte & @CRLF
		$texte = $texte & convertAlerteToLigne($liste[$i])
	Next
	Local $fichier = FileOpen($chemin, 10)
	If $fichier <> -1 Then FileWrite($fichier, $texte)
	FileClose($fichier)
EndFunc



; ~~~ Fonctions internes  la modlisation,  ne pas utiliser hors de ce fichier ~~~

; ### getStructureAlerteArray()>matrix :
; Retourne une matrice contenant les attributs et leurs formats, pour structurer un objet "Alerte"
Func getStructureAlerteArray()
	Local $structure[7][3] = [["int","index",""], ["char","dateA",16], ["char","dateB",16], ["char","periodic",16], ["char","title",64], ["char","descript",256], ["char","occurrences",512]]
;~ 	Pour dateA et dateB : StringLen("JJ/MM/AAAA HH:MM") = 16
 	Return $structure
EndFunc
; ### getStructureAlerteNumber()>int :
; Retourne le nombre d'attributs ncessaires pour structurer un objet "Alerte"
Func getStructureAlerteNumber()
	Return UBound(getStructureAlerteArray(),1)
EndFunc
; ### getStructureAlerte()>string :
; Retourne une chaine de caractres contenant les attributs et leurs formats, pour structurer un objet "Alerte"
Func getStructureAlerte()
	Local $i = 0, $string = "", $structure = getStructureAlerteArray()
	For $i=0 To getStructureAlerteNumber()-1
		If $string <> "" Then $string &= ";"
		$string &= $structure[$i][0] & " " & $structure[$i][1] & cond($structure[$i][2]<>"", "["&$structure[$i][2]&"]", "")
	Next
	Return $string
EndFunc
; ### getAlerteAttribute($alerte,$attribut)>string :
; Retourne l'attribut $attribut de l'objet "Alerte" $alerte
Func getAlerteAttribute($alerte, $attribut)
	Return DllStructGetData($alerte, $attribut)
EndFunc
; ### setAlerteAttribute($alerte,$attribut,$valeur)>void :
; Ecrit la valeur $valeur dans l'attribut $attribut de l'objet "Alerte" $alerte
Func setAlerteAttribute(ByRef $alerte, $attribut, $valeur)
	DllStructSetData($alerte, $attribut, StringReplace(StringReplace(StringReplace($valeur,"","#"),@CR,""),@LF,""))
EndFunc
; ### copyAlerteAttribute($newAlerte,$oldAlerte,$attribut)>void :
; Ecrit la valeur de l'attribut $attribut de l'objet "Alerte" $oldAlerte dans celui de l'objet "Alerte" $newAlerte
Func copyAlerteAttribute(ByRef $newAlerte, $oldAlerte, $attribut)
	setAlerteAttribute($newAlerte, $attribut, getAlerteAttribute($oldAlerte, $attribut))
EndFunc
; ### changeDateFormat($date)>string :
; Retourne une date transforme du format "JJ/MM/YYYY HH:MM" ou "YYYYMMJJHHMMSS", au format "YYYY/MM/JJ HH:MM:SS"
Func changeDateFormat($date)
	If StringRegExp($date,"^\d{4}/\d{2}/\d{2} \d{2}:\d{2}:\d{2}$") Then
		Return $date
	ElseIf StringRegExp($date,"^\d{2}/\d{2}/\d{4} \d{2}:\d{2}$") Then
		Local $split = StringSplit($date&":00","/ ")
		Return $split[3]&"/"&$split[2]&"/"&$split[1]&" "&$split[4]
	ElseIf StringRegExp($date,"^\d{14}$") Then
		Local $split = StringSplit($date,"")
		Return "" _
			& $split[1]&$split[2]&$split[3]&$split[4] & "/" & $split[5]&$split[6] & "/" & $split[7]&$split[8] _
			& " " _
			& $split[9]&$split[10] & ":" & $split[11]&$split[12] & ":" & $split[13]&$split[14]
	EndIf
	Return ""
EndFunc

#EndRegion

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
