#Include-Once

; Librairies nAiO
#Include "..\Encryptage Decryptage.au3"

#Region > Modlisation des objets "Compte"

; ~~~ Prsentation de la structure ~~~

; L'objet "Compte" est une reprsentation d'un compte de jeu Dofus; lui sont rattachs les attributs "privs" suivants :
;  - index  [int]        : indice permettant un accs rapide au compte. les valeurs de "index" doivent tre diffrents deux  deux pour chaque compte existant
;  - alias  [string-32]  : nom du compte facilitant la reconnaissance par l'utilisateur (diffrent du login de compte Dofus)
;  - avatar [string-256] : chemin du fichier dterminant l'image du compte (chemin local de prfrence)
;  - login  [string-256] : login du compte Dofus, encrypt par une clef personnelle (format : '^0x[0-9A-F]+$')
;  - pass   [string-256] : mot de passe du compte Dofus, encrypt par une clef personnelle (format : '^0x[0-9A-F]+$')
; La liste de comptes est une reprsentation des comptes enregistrs; lui est rattache la modlisation suivante :
;  - Array de taille n+1, o n reprsente le nombre de comptes enregistrs
;    - [indice 0] : valeur gale au nombre de comptes enregistrs, soit n
;    - [indice 1] : premier compte (objet "Compte")
;    - ...
;    - [indice n] : (n)ime compte (objet "Compte")
;  - Une liste vierge de comptes est donc reprsente ainsi : Dim $array[1] = [0]



; ~~~ Constructeurs et fonctions de base ~~~

; ### newCompte($index,$alias,$avatar,$login,$pass,$key)>[Compte] :
; Construit un objet "Compte"  partir des donnes $index, $alias, $avatar, $login, $pass et $key, et retourne sa valeur
; => Tous les paramtres sont tous optionnels
Func newCompte($index="", $alias="", $avatar="", $login="", $pass="", $key=$DEFAULT_KEY)
	Local $compte = DllStructCreate(getStructureCompte())
	If $index <> "" Then setCompteIndex($compte, $index)
	setCompteAlias($compte, $alias)
	setCompteAvatar($compte, $avatar)
	setCompteLogin($compte, $login, $key)
	setComptePass($compte, $pass, $key)
	Return $compte
EndFunc
; ### copyCompte($oldCompte)>[Compte] :
; Construit un objet "Compte"  partir des donnes dupliques d'un autre objet "Compte", et retourne la nouvelle valeur
Func copyCompte($oldCompte)
	Local $newCompte = newCompte(), $structure = getStructureCompteArray()
	For $i = 0 To getStructureCompteNumber()-1
		copyCompteAttribute($newCompte, $oldCompte, $structure[$i][1])
	Next
	Return $newCompte
EndFunc
; ### deleteCompte($compte)>void :
; Detruit l'objet "Compte" $compte
Func deleteCompte(ByRef $compte)
	$compte = 0
EndFunc
; ### isCompte($compte)>boolean :
; Retourne la valeur True si $compte est bien un objet "Compte", False sinon
Func isCompte($compte)
	Local $i = 0, $structure = getStructureCompteArray()
	If Not IsDllStruct($compte) Then
		Return False
	EndIf
	For $i=0 To getStructureCompteNumber()-1
		Local $valeur = DllStructGetData($compte, $structure[$i][1])
		If @error Then Return False
	Next
	Return True
EndFunc
; ### toStringCompte($compte)>string :
; Retourne une chaine de caractres identifiant un objet "Compte"
Func toStringCompte($compte)
	Return getCompteIndex($compte)&" - "&getCompteAlias($compte)
EndFunc
; ### newListeComptes()>array :
; Cre une liste vierge de comptes
Func newListeComptes()
	Local $retour[1]
	setListeComptesLength($retour,0)
	Return $retour
EndFunc
; ### deleteListeComptes($liste)>void :
; Ecrase $liste avec une liste vierge d'objets "Compte" aprs avoir dtruit chaque objet "Compte" prsent
Func deleteListeComptes(ByRef $liste)
	If IsArray($liste) Then
		For $i = 0 To UBound($liste)-1
			$liste[$i] = 0
		Next
	EndIf
	$liste = newListeComptes()
EndFunc



; ~~~ Fonctions de dbugage ~~~

; ### displayCompte($compte)>void :
; Affiche les informations de $compte dans une fentre type MsgBox
Func displayComptes($liste)
	For $i = 1 To getListeComptesLength($liste)
		displayCompte(getCompteInListAt($liste,$i),$i&"/"&getListeComptesLength($liste))
	Next
EndFunc
; ### displayComptes($liste)>void :
; Afficher les informations des lments de $liste dans des fentres ordonnes type MsgBox
Func displayCompte($compte, $title="")
	Local $fullTitle = "Compte"
	If $title<>"" Then $fullTitle = $fullTitle & " ["&$title&"]"
	If isCompte($compte) Then
		Local $structure = getStructureCompteArray(), $msg = _
			"Compte.toString: '" & toStringCompte($compte) & "'" & @CRLF & _
			"Struct Size: " & DllStructGetSize($compte) & @CRLF & _
			"Struct pointer: " & DllStructGetPtr($compte) & @CRLF & _
			"Data:"
		For $i = 0 To getStructureCompteNumber()-1
			$msg &= @CRLF & @TAB & $structure[$i][1]&": '"&getCompteAttribute($compte, $structure[$i][1])&"'"
		Next
		MsgBox(0, $fullTitle, "{"&@CRLF&$msg&@CRLF&"}" )
	Else
		MsgBox(16, $fullTitle, "!!! Ce compte n'est pas un objet conforme !!!" )
	EndIf
EndFunc



; ~~~ Fonctions Get/Set pour les attributs index, alias et avatar ~~~

; ### getCompteIndex($compte)>string :
; Lit l'attribut index de $compte
Func getCompteIndex($compte)
	Return getCompteAttribute($compte, "index")
EndFunc
; ### setCompteIndex($compte,$index)>void :
; Ecrit la valeur $index dans l'attribut index de $compte
Func setCompteIndex(ByRef $compte, $index)
	setCompteAttribute($compte, "index", $index)
EndFunc
; ### getCompteAlias($compte)>string :
; Lit l'attribut alias de $compte
Func getCompteAlias($compte)
	Return getCompteAttribute($compte, "alias")
EndFunc
; ### setCompteAlias($compte,$alias)>void :
; Ecrit la valeur $alias dans l'attribut alias de $compte
Func setCompteAlias(ByRef $compte, $alias)
	setCompteAttribute($compte, "alias", $alias)
EndFunc
; ### getCompteAvatar($compte)>string :
; Lit l'attribut avatar de $compte
Func getCompteAvatar($compte)
	Return getCompteAttribute($compte, "avatar")
EndFunc
; ### setCompteAvatar($compte,$avatar)>void :
; Ecrit la valeur $avatar dans l'attribut avatar de $compte
Func setCompteAvatar(ByRef $compte, $avatar)
	setCompteAttribute($compte, "avatar", $avatar)
EndFunc



; ~~~ Fonctions Get/Set/Is modifies pour les attributs login et pass ~~~

; ### getCompteLogin($compte,$key)>string :
; Lit la valeur dcrypte avec $key de l'attribut login de $compte
; => $key peut tre une chaine vide, dans ce cas, la valeur retourne est l'attribut login de $compte, sans dcryptage pralable
; => Le paramtre $key est optionnel, dans le cas o il est omis, on utilise $DEFAULT_Key pour le dcryptage
; => La fonction retourne une chaine vide si le dcryptage a chou
Func getCompteLogin($compte, $key=$DEFAULT_Key)
	If $key = "" Then
		Return getCompteAttribute($compte, "login")
	Else
		Return decryptage(getCompteLogin($compte,""), $key)
	EndIf
EndFunc
; ### setCompteLogin($compte,$login,$key)>void :
; Ecrit la valeur $login encrypte avec $key dans l'attribut login de $compte
; => $key peut tre une chaine vide, dans ce cas, la valeur crite est la variable $login, sans encryptage pralable
; => Le paramtre $key est optionnel, dans le cas o il est omis, on utilise $DEFAULT_Key pour l'encryptage
Func setCompteLogin(ByRef $compte, $login, $key=$DEFAULT_Key)
	If $key = "" Then
		setCompteAttribute($compte, "login", $login)
	Else
		setCompteLogin($compte, encryptage($login,$key), "")
	EndIf
EndFunc
; ### getComptePass($compte,$key)>string :
; Lit la valeur dcrypte avec $key de l'attribut pass de $compte
; => $key peut tre une chaine vide, dans ce cas, la valeur retourne est l'attribut pass de $compte, sans dcryptage pralable
; => Le paramtre $key est optionnel, dans le cas o il est omis, on utilise $DEFAULT_Key pour le dcryptage
; => La fonction retourne une chaine vide si le dcryptage a chou
Func getComptePass($compte, $key=$DEFAULT_Key)
	If $key = "" Then
		Return getCompteAttribute($compte, "pass")
	Else
		Return decryptage(getComptePass($compte,""), $key)
	EndIf
EndFunc
; ### setComptePass($compte,$pass,$key)>void :
; Ecrit la valeur $pass encrypte avec $key dans l'attribut pass de $compte
; => $key peut tre une chaine vide, dans ce cas, la valeur crite est la variable $pass, sans encryptage pralable
; => Le paramtre $key est optionnel, dans le cas o il est omis, on utilise $DEFAULT_Key pour l'encryptage
Func setComptePass(ByRef $compte, $pass, $key=$DEFAULT_Key)
	If $key = "" Then
		setCompteAttribute($compte, "pass", $pass)
	Else
		setComptePass($compte, encryptage($pass,$key), "")
	EndIf
EndFunc
; ### isCompteLockedByKey($compte)>boolean :
; Retourne la valeur True si le compte $compte utilise une clef pour verrouiller les attributs login et pass, False sinon
Func isCompteLockedByKey($compte)
	Return Not checkCompteWithKey($compte,$DEFAULT_Key)
EndFunc
; ### checkCompteWithKey($compte,$key)>boolean :
; Retourne la valeur True si le compte $compte utilise la clef $key pour verrouiller les attributs login et pass, False sinon
Func checkCompteWithKey($compte,$key)
	If $key = "" Then Return False
	Return (getCompteLogin($compte,"")=encryptage("",$key) Or getCompteLogin($compte,$key) <> "") And (getComptePass($compte,"")=encryptage("",$key) Or getComptePass($compte,$key) <> "")
EndFunc



; ~~~ Fonctions de listing de comptes ~~~

; ### addToListeComptes($liste, $compte)>void :
; Ajoute l'objet "Compte" $compte  la fin de la liste de comptes $liste
; => La valeur  l'indice 0 est incrmente automatiquement si un objet "Compte" ayant la mme valeur index existe dj dans la liste
Func addToListeComptes(ByRef $liste, $compte)
	Local $indexCompte = getCompteIndex($compte)
	If isCompte(getCompteInListFromIndex($liste,$indexCompte)) Then
		setCompteIndex($compte, $indexCompte+1)
		addToListeComptes($liste, $compte)
	Else
		setListeComptesLength($liste,getListeComptesLength($liste)+1)
		setCompteInListAt($liste,getListeComptesLength($liste),$compte)
	EndIf
EndFunc
; ### delInListeComptesAt($liste,$i)>[Compte] :
; Efface l'objet "Compte" prsent  l'indice $i de la liste de comptes $liste, et retourne l'objet "Compte" qui a t retir
; => Si l'indice $i n'est pas sens contenir un objet "Compte" ($i<1 ou $i>n), la chaine vide "" est retourne
Func delInListeComptesAt(ByRef $liste, $i)
	Local $deleted = ""
	If $i>=1 And $i<=getListeComptesLength($liste) Then
		$deleted = getCompteInListAt($liste,$i)
		For $j = $i To getListeComptesLength($liste)-1
			setCompteInListAt($liste,$j,getCompteInListAt($liste,$j+1))
		Next
		setListeComptesLength($liste,getListeComptesLength($liste)-1)
	EndIf
	Return $deleted
EndFunc
; ### delInListeComptesFromToString($liste,$toStringCompte)>[Compte] :
; Efface l'objet "Compte" ayant $toStringCompte comme identifiant, de la liste de comptes $liste, et retourne l'objet "Compte" qui a t retir
; => Si aucun objet "Compte" ne correspond au critre, la chaine vide "" est retourne
Func delInListeComptesFromToString(ByRef $liste, $toStringCompte)
	For $i = 1 To getListeComptesLength($liste)
		If toStringCompte(getCompteInListAt($liste,$i)) = $toStringCompte Then ExitLoop
	Next
	If $i <= getListeComptesLength($liste) Then Return delInListeComptesAt($liste, $i)
	Return ""
EndFunc
; ### moveInListeComptesFromToString($liste, $toStringCompte1, $toStringCompte2)>void :
; Retire l'objet "Compte" ayant $toStringCompte1 comme identifiant, de la liste de comptes $liste, et dplace les objets "Compte"
; dans la liste afin que l'objet "Compte" retir puisse tre  la place de celui ayant $toStringCompte2 comme identifiant
; => Si aucun objet "Compte" ne correspond au critre $toStringCompte1 ou $toStringCompte2, aucune action n'est effectue
Func moveInListeComptesFromToString(ByRef $liste, $toStringCompte1, $toStringCompte2)
	For $i = 1 To getListeComptesLength($liste)
		If toStringCompte(getCompteInListAt($liste,$i)) = $toStringCompte1 Then ExitLoop
	Next
	For $j = 1 To getListeComptesLength($liste)
		If toStringCompte(getCompteInListAt($liste,$j)) = $toStringCompte2 Then ExitLoop
	Next
	If $i <> $j And $i <= getListeComptesLength($liste) And $j <= getListeComptesLength($liste) Then
		Local $compteTemp = getCompteInListAt($liste,$i), $step = -1
		If $i<$j Then $step = 1
		For $k = $i To $j-$step Step $step
			setCompteInListAt($liste,$k,getCompteInListAt($liste,$k+$step))
		Next
		setCompteInListAt($liste,$j,$compteTemp)
	EndIf
EndFunc
; ### getCompteInListFromAlias($liste,$alias)>[Compte] :
; Retourne l'objet "Compte" prsent dans la liste de comptes $liste ayant son attribut alias gal  $alias
; => Si aucun objet "Compte" ne rpond au critre, la chaine vide "" est retourne
Func getCompteInListFromAlias($liste,$alias)
	For $i = 1 To getListeComptesLength($liste)
		If isCompte(getCompteInListAt($liste,$i)) And getCompteAlias(getCompteInListAt($liste,$i))=$alias Then Return getCompteInListAt($liste,$i)
	Next
	Return ""
EndFunc
; ### getCompteInListFromIndex($liste,$index)>[Compte] :
; Retourne l'objet "Compte" prsent dans la liste de comptes $liste ayant son attribut index gal  $index
; => Si aucun objet "Compte" ne rpond au critre, la chaine vide "" est retourne
Func getCompteInListFromIndex($liste,$index)
	For $i = 1 To getListeComptesLength($liste)
		If isCompte(getCompteInListAt($liste,$i)) And getCompteIndex(getCompteInListAt($liste,$i))=$index Then Return getCompteInListAt($liste,$i)
	Next
	Return ""
EndFunc
; ### getCompteInListFromToString($liste,$toString)>[Compte] :
; Retourner l'objet "Compte" prsent dans la liste de comptes $liste ayant $toStringCompte comme identifiant
; => Si aucun objet "Compte" ne rpond au critre, la chaine vide "" est retourne
Func getCompteInListFromToString($liste,$toStringCompte)
	For $i = 1 To getListeComptesLength($liste)
		If isCompte(getCompteInListAt($liste,$i)) And toStringCompte(getCompteInListAt($liste,$i))=$toStringCompte Then Return getCompteInListAt($liste,$i)
	Next
	Return ""
EndFunc
; ### setCompteInListFromToString($liste,$toStringCompte,$alias,$avatar,$login,$pass,$key)>[Compte] :
; Modifie les informations de l'objet "Compte" prsent dans la liste de comptes $liste ayant $toStringCompte comme identifiant
; par les valeurs $alias, $avatar, $login, $pass et $key, et retourne la valeur de l'objet "Compte" avant modification
; => Si aucun objet "Compte" ne rpond au critre, la chaine vide "" est retourne
Func setCompteInListFromToString(ByRef $liste, $toStringCompte, $alias,$avatar,$login,$pass,$key=$DEFAULT_Key)
	Local $modified = ""
	For $i = 1 To getListeComptesLength($liste)
		If isCompte(getCompteInListAt($liste,$i)) And toStringCompte(getCompteInListAt($liste,$i))=$toStringCompte Then
			Local $compte = getCompteInListAt($liste,$i)
			$modified = copyCompte($compte)
			setCompteAlias($compte,$alias)
			setCompteAvatar($compte,$avatar)
			setCompteLogin($compte,$login,$key)
			setComptePass($compte,$pass,$key)
		EndIf
	Next
	Return $modified
EndFunc
; ### getCompteInListAt($liste,$i)>[Compte] :
; Retourne l'objet "Compte" prsent  l'indice $i dans la liste de comptes $liste
; => Si l'indice $i ne contient pas un objet "Compte", la chaine vide "" est retourne
Func getCompteInListAt($liste,$i)
	If isCompte($liste[$i]) Then
		Return $liste[$i]
	Else
		Return ""
	EndIf
EndFunc
; ### setCompteInListAt($liste,$i,$compte)>void :
; Ecrit l'objet "Compte" $compte  l'indice $i de la liste de comptes $liste
; => Si l'indice $i est invalide, aucune action n'est effectue
Func setCompteInListAt(ByRef $liste,$i,$compte)
	If $i>0 And $i<=getListeComptesLength($liste) Then
		$liste[$i] = $compte
	EndIf
EndFunc
; ### getListeComptesLength($liste)>int :
; Retourne le nombre d'objets "Compte" prsent dans la liste de comptes $liste
Func getListeComptesLength($liste)
	Return $liste[0]
EndFunc
; ### setListeComptesLength($liste,$length)>void :
; Redimensionne  $length la taille de la liste de comptes $liste
Func setListeComptesLength(ByRef $liste, $length)
	If $length>=0 Then
		$liste[0] = $length
		ReDim $liste[getListeComptesLength($liste)+1]
	EndIf
EndFunc



; ~~~ Fonctions pour mashall/unmarshall ~~~

; ### convertLigneToCompte($ligne,$pass)>[Compte] :
; Transforme la chaine de caractres $ligne en objet "Compte" (unmarshall)
; => $pass peut tre une chaine vide, dans ce cas, la valeur est convertie sans dcryptage pralable
Func convertLigneToCompte($ligne, $pass="")
	Local $ligneToProcess = decryptage($ligne,$pass)
	If $pass="" Then $ligneToProcess = $ligne
	Local $param = StringSplit($ligneToProcess,"")
	If $param[0] = getStructureCompteNumber() Then Return newCompte($param[1],$param[2],$param[3],$param[4],$param[5],"")
	SetError(1)
EndFunc
; ### convertCompteToLigne($compte,$pass)>string :
; Srialise l'objet "Compte" $compte en chaine de caractres (marshall)
; => $pass peut tre une chaine vide, dans ce cas, la valeur est convertie sans encryptage pralable
Func convertCompteToLigne($compte, $pass="")
	Local $ligne = getCompteIndex($compte)&""&getCompteAlias($compte)&""&getCompteAvatar($compte)&""&getCompteLogin($compte,"")&""&getComptePass($compte,"")
	Local $ligneToProcess = encryptage($ligne,$pass)
	If $pass="" Then $ligneToProcess = $ligne
	Return $ligneToProcess
EndFunc
; ### convertFichierToListeComptes($chemin,$liste)>void :
; Lit les donnes du fichier de config $chemin, et enregistre la liste de comptes correspondante dans $liste
; => Les donnes de $liste sont crases
Func convertFichierToListeComptes($chemin, ByRef $liste, $pass="")
	$liste = newListeComptes()
	Local $fichier = FileOpen($chemin, 0)
	While $fichier <> -1
		Local $ligne = FileReadLine($fichier)
		If @error <> 0 Then ExitLoop
		Local $compte = convertLigneToCompte($ligne, $pass)
		If @error = 0 Then addToListeComptes($liste, $compte)
	WEnd
	FileClose($fichier)
EndFunc
; ### convertListeComptesToFichier($liste,$chemin)>void :
; Lit les donnes de la liste de comptes $liste et enregistre les donnes correspondantes dans le fichier de config $chemin
; => Les donnes du fichier $chemin sont crases
Func convertListeComptesToFichier($liste, $chemin, $pass="")
	Local $texte = ""
	For $i = 1 To getListeComptesLength($liste)
		If $texte <> "" Then $texte = $texte & @CRLF
		$texte = $texte & convertCompteToLigne(getCompteInListAt($liste,$i), $pass)
	Next
	Local $fichier = FileOpen($chemin, 10)
	If $fichier <> -1 Then FileWrite($fichier, $texte)
	FileClose($fichier)
EndFunc



; ~~~ Fonctions internes  la modlisation,  ne pas utiliser hors de ce fichier ~~~

; ### getStructureCompteArray()>matrix :
; Retourne une matrice contenant les attributs et leurs formats, pour structurer un objet "Compte"
Func getStructureCompteArray()
	Local $structure[5][3] = [["int","index",""], ["char","alias",32], ["char","avatar",256], ["char","login",256], ["char","pass",256]]
	Return $structure
EndFunc
; ### getStructureCompteNumber()>int :
; Retourne le nombre d'attributs ncessaires pour structurer un objet "Compte"
Func getStructureCompteNumber()
	Return UBound(getStructureCompteArray(),1)
EndFunc
; ### getStructureCompte()>string :
; Retourne une chaine de caractres contenant les attributs et leurs formats, pour structurer un objet "Compte"
Func getStructureCompte()
	Local $i = 0, $string = "", $structure = getStructureCompteArray()
	For $i=0 To getStructureCompteNumber()-1
		If $string <> "" Then $string &= ";"
		$string &= $structure[$i][0] & " " & $structure[$i][1]
		If $structure[$i][2]<>"" Then $string &= "["&$structure[$i][2]&"]"
	Next
	Return $string
EndFunc
; ### getCompteAttribute($compte,$attribut)>string :
; Retourne l'attribut $attribut de l'objet "Compte" $compte
Func getCompteAttribute($compte, $attribut)
	Return DllStructGetData($compte, $attribut)
EndFunc
; ### setCompteAttribute($compte,$attribut,$valeur)>void :
; Ecrit la valeur $valeur dans l'attribut $attribut de l'objet "Compte" $compte
Func setCompteAttribute(ByRef $compte, $attribut, $valeur)
	DllStructSetData($compte, $attribut, StringReplace($valeur,"","#"))
EndFunc
; ### copyCompteAttribute($newCompte,$oldCompte,$attribut)>void :
; Ecrit la valeur de l'attribut $attribut de l'objet "Compte" $oldCompte dans celui de l'objet "Compte" $newCompte
Func copyCompteAttribute(ByRef $newCompte, $oldCompte, $attribut)
	setCompteAttribute($newCompte, $attribut, getCompteAttribute($oldCompte, $attribut))
EndFunc

#EndRegion

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; - NB - :
; Chaine en claire de taille maximale 106 caractres
; (107 caractres clairs => 258 caractres encrypts > 256 autoriss)