#Include-Once

Global $test[1] = [0]

#Region > Modlisation des objets "Lien"

; ~~~ Prsentation de la structure ~~~

; L'objet "Lien" est une reprsentation d'un lment (fichier, programme ou dossier) du disque dur dans le nAiO; lui sont rattachs les attributs "privs" suivants :
;  - index  [entier]      : indice permettant un accs rapide au lien. Les valeurs de "index" doivent tre diffrents deux  deux pour chaque lien existant
;  - alias  [string-64]   : nom unique associ au lien
;  - chemin [string-1024] : chemin du lien
;  - type   [string-1]    : type d'opration sur le lien (F:fichier, P:programme, D:dossier)
; La liste de liens est une reprsentation des liens enregistrs; lui est rattache la modlisation suivante :
;  - Array de taille n+1, o n reprsente le nombre de liens enregistrs
;    - [indice 0] : valeur gale au nombre de liens enregistrs, soit n
;    - [indice 1] : premier lien (objet "Lien")
;    - ...
;    - [indice n] : (n)ime lien (objet "Lien")
;  - Une liste vierge de liens est donc reprsente ainsi : Dim $array[1] = [0]



; ~~~ Constructeurs et fonctions de base ~~~

; ### newLien($index,$alias,$chemin,$type)>[Lien] :
; Construit un objet "Lien"  partir des donnes $index, $alias, $chemin et $type, et retourne sa valeur
; => Tous les paramtres sont tous optionnels
Func newLien($index="", $alias="", $chemin="", $type="")
	Local $lien = DllStructCreate(getStructureLien())
	If $index <> "" Then setLienIndex($lien, $index)
	setLienAlias($lien, $alias)
	setLienChemin($lien, $chemin)
	setLienType($lien, $type)
	Return $lien
EndFunc
; ### copyLien($oldLien)>[Lien] :
; Construit un objet "Lien"  partir des donnes dupliques d'un autre objet "Lien", et retourne la nouvelle valeur
Func copyLien($oldLien)
	Local $newLien = newLien(), $structure = getStructureLienArray()
	For $i = 0 To getStructureLienNumber()-1
		copyLienAttribute($newLien, $oldLien, $structure[$i][1])
	Next
	Return $newLien
EndFunc
; ### deleteLien($lien)>void :
; Detruit l'objet "Lien" $lien
Func deleteLien(ByRef $lien)
	$lien = 0
EndFunc
; ### isLien($lien)>boolean :
; Retourne la valeur True si $lien est bien un objet "Lien", False sinon
Func isLien($lien)
	Local $i = 0, $valeur = "", $structure = getStructureLienArray()
	If Not IsDllStruct($lien) Then
		Return False
	EndIf
	For $i=0 To UBound($structure)-1
		Local $valeur = DllStructGetData($lien, $structure[$i][1])
		If @error Then Return False
	Next
	Return True
EndFunc
; ### toStringLien($lien)>string :
; Retourne une chaine de caractres identifiant un objet "Lien"
Func toStringLien($lien)
	Return getLienIndex($lien)&" - "&getLienAlias($lien)
EndFunc
; ### newListeLiens()>array :
; Cre une liste vierge de liens
Func newListeLiens() ; => tableau
	Local $retour[1]
	setListeLiensLength($retour,0)
	Return $retour
EndFunc
; ### deleteListeLiens($liste)>void :
; Ecrase $liste avec une liste vierge d'objets "Lien" aprs avoir dtruit chaque objet "Lien" prsent
Func deleteListeLiens(ByRef $liste)
	If IsArray($liste) Then
		For $i = 0 To UBound($liste)-1
			$liste[$i] = 0
		Next
	EndIf
	$liste = newListeLiens()
EndFunc



; ~~~ Fonctions de dbugage ~~~

; ### displayLiens($liste)>void :
; Afficher les informations des lments de $liste dans des fentres ordonnes type MsgBox
Func displayLiens($liste)
	For $i = 1 To _Min(UBound($liste)-1,$liste[0])
		displayLien($liste[$i],$i&"/"&$liste[0]&cond($liste[0]=UBound($liste)-1,""," (TAILLE:"&UBound($liste)-1&")"))
	Next
EndFunc
; ### displayLien($lien,$title)>void :
; Affiche les informations de $lien dans une fentre type MsgBox
Func displayLien($lien, $title="")
	Local $fullTitle = "Lien"
	If $title<>"" Then $fullTitle = $fullTitle & " ["&$title&"]"
	If isLien($lien) Then
		Local $structure = getStructureLienArray(), $msg = _
			"Struct Size: " & DllStructGetSize($lien) & @CRLF & _
			"Struct pointer: " & DllStructGetPtr($lien) & @CRLF & _
			"Data:"
		For $i = 0 To getStructureLienNumber()-1
			$msg &= @CRLF & @TAB & $structure[$i][1]&": '"&getLienAttribute($lien, $structure[$i][1])&"'"
		Next
		MsgBox(0, $fullTitle, "{"&@CRLF&$msg&@CRLF&"}" )
	Else
		MsgBox(16, $fullTitle, "!!! Ce lien n'est pas un objet conforme !!!" )
	EndIf
EndFunc



; ~~~ Fonctions Get/Set pour les attributs index, alias, chemin et type ~~~

; ### getLienIndex($lien)>string :
; Lit l'attribut index de $lien
Func getLienIndex($lien)
	Return getLienAttribute($lien, "index")
EndFunc
; ### setLienIndex($lien,$index)>void :
; Ecrit la valeur $index dans l'attribut index de $lien
Func setLienIndex(ByRef $lien, $index)
	setLienAttribute($lien, "index", $index)
EndFunc
; ### getLienAlias($lien)>string :
; Lit l'attribut alias de $lien
Func getLienAlias($lien)
	Return getLienAttribute($lien, "alias")
EndFunc
; ### setLienAlias($lien,$alias)>void :
; Ecrit la valeur $alias dans l'attribut alias de $lien
Func setLienAlias(ByRef $lien, $alias)
	setLienAttribute($lien, "alias", $alias)
EndFunc
; ### getLienChemin($lien)>string :
; Lit l'attribut chemin de $lien
Func getLienChemin($lien)
	Return getLienAttribute($lien, "chemin")
EndFunc
; ### setLienChemin($lien,$chemin)>void :
; Ecrit la valeur $chemin dans l'attribut chemin de $lien
Func setLienChemin(ByRef $lien, $chemin)
	setLienAttribute($lien, "chemin", $chemin)
EndFunc
; ### getLienType($lien)>string :
; Lit l'attribut type de $lien
Func getLienType($lien)
	Return getLienAttribute($lien, "type")
EndFunc
; ### isLienTypeDossier($lien)>boolean :
; Vrifie si l'attribut type de $lien est  "D"
Func isLienTypeDossier($lien)
	Return (getLienType($lien) = "D")
EndFunc
; ### isLienTypeFichier($lien)>boolean :
; Vrifie si l'attribut type de $lien est  "F"
Func isLienTypeFichier($lien)
	Return (getLienType($lien) = "F")
EndFunc
; ### isLienTypeProgramme($lien)>boolean :
; Vrifie si l'attribut type de $lien est  "P"
Func isLienTypeProgramme($lien)
	Return (getLienType($lien) = "P")
EndFunc
; ### setLienType($lien,$type)>void :
; Ecrit la valeur $type dans l'attribut type de $lien
Func setLienType(ByRef $lien, $type)
	setLienAttribute($lien, "type", StringUpper($type))
EndFunc
; ### setLienTypeDossier($lien)>void :
; Ecrit la valeur "D" dans l'attribut type de $lien
Func setLienTypeDossier(ByRef $lien)
	setLienAttribute($lien, "type", "D")
EndFunc
; ### setLienTypeFichier($lien)>void :
; Ecrit la valeur "F" dans l'attribut type de $lien
Func setLienTypeFichier(ByRef $lien)
	setLienAttribute($lien, "type", "F")
EndFunc
; ### setLienTypeProgramme($lien)>void :
; Ecrit la valeur "P" dans l'attribut type de $lien
Func setLienTypeProgramme(ByRef $lien)
	setLienAttribute($lien, "type", "P")
EndFunc



; ~~~ Fonctions de listing d'liens ~~~

; ### addToListeLiens($liste,$lien,$force)>void :
; Ajoute l'objet "Lien" $lien  la fin de la liste de liens $liste
; => La valeur  l'indice 0 est incrmente automatiquement si un objet "Lien" ayant la mme valeur index existe dj dans la liste
; => Avec $force gal  False, l'objet $lien n'est pas ajout  la liste si un objet "Lien" ayant la mme valeur index y existe dj
; => Le parametre $force est optionnel
Func addToListeLiens(ByRef $liste, $lien, $force = True)
	Local $indexLien = getLienIndex($lien)
	If isLien(getLienInListFromIndex($liste,$indexLien)) Then
		If $force Then
			setLienIndex($lien, $indexLien+1)
			addToListeLiens($liste, $lien)
		EndIf
	Else
		setListeLiensLength($liste,getListeLiensLength($liste)+1)
		setLienInListAt($liste,getListeLiensLength($liste),$lien)
	EndIf
EndFunc
; ### delInListeLiensAt($liste,$i)>[Lien] :
; Efface l'objet "Lien" prsent  l'indice $i de la liste de liens $liste, et retourne l'objet "Lien" qui a t retir
; => Si l'indice $i n'est pas sens contenir un objet "Lien" ($i<1 ou $i>n), la chaine vide "" est retourne
Func delInListeLiensAt(ByRef $liste, $i)
	Local $deleted = ""
	If $i>=1 And $i<=getListeLiensLength($liste) Then
		$deleted = getLienInListAt($liste,$i)
		For $j = $i To getListeLiensLength($liste)-1
			setLienInListAt($liste,$j,getLienInListAt($liste,$j+1))
		Next
		setListeLiensLength($liste,getListeLiensLength($liste)-1)
	EndIf
	Return $deleted
EndFunc
; ### delInListeLiensFromToString($liste,$toStringLien)>[Lien] :
; Efface l'objet "Lien" ayant $toStringLien comme identifiant, de la liste de liens $liste, et retourne l'objet "Lien" qui a t retir
; => Si aucun objet "Lien" ne correspond au critre, la chaine vide "" est retourne
Func delInListeLiensFromToString(ByRef $liste, $toStringLien)
	Local $i = 1, $deleted = ""
	While $i <= $liste[0]
		If toStringLien($liste[$i]) = $toStringLien Then ExitLoop
		$i = $i + 1
	WEnd
	If $i <= $liste[0] Then
		$deleted = $liste[$i]
		For $j = $i To $liste[0]-1
			$liste[$j] = $liste[$j+1]
		Next
		$liste[0] = $liste[0]-1
		ReDim $liste[$liste[0]+1]
	EndIf
	Return $deleted
EndFunc
; ### moveInListeLiensFromToString($liste, $toStringLien1, $toStringLien2)>void :
; Retire l'objet "Lien" ayant $toStringLien1 comme identifiant, de la liste de liens $liste, et dplace les objets "Lien"
; dans la liste afin que l'objet "Lien" retir puisse tre  la place de celui ayant $toStringLien2 comme identifiant
; => Si aucun objet "Lien" ne correspond au critre $toStringLien1 ou $toStringLien2, aucune action n'est effectue
Func moveInListeLiensFromToString(ByRef $liste, $toStringLien1, $toStringLien2)
	For $i = 1 To getListeLiensLength($liste)
		If toStringLien(getLienInListAt($liste,$i)) = $toStringLien1 Then ExitLoop
	Next
	For $j = 1 To getListeLiensLength($liste)
		If toStringLien(getLienInListAt($liste,$j)) = $toStringLien2 Then ExitLoop
	Next
	If $i <> $j And $i <= getListeLiensLength($liste) And $j <= getListeLiensLength($liste) Then
		Local $lienTemp = getLienInListAt($liste,$i), $step = -1
		If $i<$j Then $step = 1
		For $k = $i To $j-$step Step $step
			setLienInListAt($liste,$k,getLienInListAt($liste,$k+$step))
		Next
		setLienInListAt($liste,$j,$lienTemp)
	EndIf
EndFunc
; ### getLienInListFromIndex($liste,$index)>[Lien] :
; Retourne l'objet "Lien" prsent dans la liste de liens $liste ayant son attribut index gal  $index
; => Si aucun objet "Lien" ne rpond au critre, la chaine vide "" est retourne
Func getLienInListFromIndex($liste,$index)
	For $i = 0 To UBound($liste)-1
		If isLien($liste[$i]) And getLienIndex($liste[$i])=$index Then
			Return $liste[$i]
		EndIf
	Next
	Return "" ; Si rien trouv
EndFunc
; ### getLienInListFromToString($liste,$toString)>[Lien] :
; Retourner l'objet "Lien" prsent dans la liste de liens $liste ayant $toStringLien comme identifiant
; => Si aucun objet "Lien" ne rpond au critre, la chaine vide "" est retourne
Func getLienInListFromToString($liste,$toString)
	For $i = 0 To UBound($liste)-1
		If isLien($liste[$i]) And toStringLien($liste[$i])=$toString Then
			Return $liste[$i]
		EndIf
	Next
	Return "" ; Si rien trouv
EndFunc
; ### setLienInListFromToString($liste,$toStringLien,$alias,$chemin,$type)>[Lien] :
; Modifie les informations de l'objet "Lien" prsent dans la liste de liens $liste ayant $toStringLien comme identifiant
; par les valeurs $alias, $chemin et $type, et retourne la valeur de l'objet "Lien" avant modification
; => Si aucun objet "Lien" ne rpond au critre, la chaine vide "" est retourne
Func setLienInListFromToString(ByRef $liste, $toStringLien, $alias,$chemin,$type)
	Local $modified = ""
	For $i = 0 To UBound($liste)-1
		If isLien($liste[$i]) And toStringLien($liste[$i])=$toStringLien Then
			$modified = copyLien($liste[$i])
			setLienAlias($liste[$i],$alias)
			setLienChemin($liste[$i],$chemin)
			setLienType($liste[$i],$type)
		EndIf
	Next
	Return $modified
EndFunc
; ### setLienInListAt($liste,$i,$lien)>void :
; Ecrit l'objet "Lien" $lien  l'indice $i de la liste de liens $liste
; => Si l'indice $i est invalide, aucune action n'est effectue
Func setLienInListAt(ByRef $liste,$i,$lien)
	If $i>0 And $i<=getListeLiensLength($liste) Then
		$liste[$i] = $lien
	EndIf
EndFunc
; ### getLienInListAt($liste,$i)>[Lien] :
; Retourne l'objet "Lien" prsent  l'indice $i dans la liste de liens $liste
; => Si l'indice $i ne contient pas un objet "Lien", la chaine vide "" est retourne
Func getLienInListAt($liste,$i)
	If isLien($liste[$i]) Then
		Return $liste[$i]
	Else
		Return ""
	EndIf
EndFunc
; ### getListeLiensLength($liste)>int :
; Retourne le nombre d'objets "Lien" prsent dans la liste de liens $liste
Func getListeLiensLength($liste)
	Return $liste[0]
EndFunc
; ### setListeLiensLength($liste,$length)>void :
; Redimensionne  $length la taille de la liste de liens $liste
Func setListeLiensLength(ByRef $liste, $length)
	If $length>=0 Then
		$liste[0] = $length
		ReDim $liste[getListeLiensLength($liste)+1]
	EndIf
EndFunc



; ~~~ Fonctions pour mashall/unmarshall ~~~

; ### convertLigneToLien($ligne)>[Lien] :
; Transforme la chaine de caractres $ligne en objet "Lien" (unmarshall)
Func convertLigneToLien($ligne)
	Local $param = StringSplit($ligne,"")
	If $param[0] <= getStructureLienNumber() Then Return newLien($param[1],$param[2],$param[3],$param[4])
	SetError(1)
EndFunc
; ### convertLienToLigne($lien)>string :
; Srialise l'objet "Lien" $lien en chaine de caractres (marshall)
Func convertLienToLigne($lien)
	Return getLienIndex($lien)&""&getLienAlias($lien)&""&getLienChemin($lien)&""&getLienType($lien)
EndFunc
; ### convertFichierToListeLiens($chemin,$liste)>void :
; Lit les donnes du fichier de config $chemin, et enregistre la liste de liens correspondante dans $liste
; => Les donnes de $liste sont crases
Func convertFichierToListeLiens($chemin, ByRef $liste, $pass="")
	$liste = newListeLiens()
	Local $fichier = FileOpen($chemin, 0)
	While $fichier <> -1
		Local $ligne = FileReadLine($fichier)
		If @error <> 0 Then ExitLoop
		Local $lien = convertLigneToLien($ligne)
		If @error = 0 Then addToListeLiens($liste, $lien)
	WEnd
	FileClose($fichier)
EndFunc
; ### convertListeLiensToFichier($liste,$chemin)>void :
; Lit les donnes de la liste de liens $liste et enregistre les donnes correspondantes dans le fichier de config $chemin
; => Les donnes du fichier $chemin sont crases
Func convertListeLiensToFichier($liste, $chemin, $pass="")
	Local $texte = ""
	For $i = 1 To $liste[0]
		If $texte <> "" Then $texte = $texte & @CRLF
		$texte = $texte & convertLienToLigne($liste[$i])
	Next
	Local $fichier = FileOpen($chemin, 10)
	If $fichier <> -1 Then FileWrite($fichier, $texte)
	FileClose($fichier)
EndFunc



; ~~~ Fonctions internes  la modlisation,  ne pas utiliser hors de ce fichier ~~~

; ### getStructureLienArray()>matrix :
; Retourne une matrice contenant les attributs et leurs formats, pour structurer un objet "Lien"
Func getStructureLienArray()
	Local $structure[4][3] = [["int","index",""], ["char","alias",64], ["char","chemin",1024], ["char","type",1]]
 	Return $structure
EndFunc
; ### getStructureLienNumber()>int :
; Retourne le nombre d'attributs ncessaires pour structurer un objet "Lien"
Func getStructureLienNumber()
	Return UBound(getStructureLienArray(),1)
EndFunc
; ### getStructureLien()>string :
; Retourne une chaine de caractres contenant les attributs et leurs formats, pour structurer un objet "Lien"
Func getStructureLien()
	Local $i = 0, $string = "", $structure = getStructureLienArray()
	For $i=0 To getStructureLienNumber()-1
		If $string <> "" Then $string &= ";"
		$string &= $structure[$i][0] & " " & $structure[$i][1] & cond($structure[$i][2]<>"", "["&$structure[$i][2]&"]", "")
	Next
	Return $string
EndFunc
; ### getLienAttribute($lien,$attribut)>string :
; Retourne l'attribut $attribut de l'objet "Lien" $lien
Func getLienAttribute($lien, $attribut)
	Return DllStructGetData($lien, $attribut)
EndFunc
; ### setLienAttribute($lien,$attribut,$valeur)>void :
; Ecrit la valeur $valeur dans l'attribut $attribut de l'objet "Lien" $lien
Func setLienAttribute(ByRef $lien, $attribut, $valeur)
	DllStructSetData($lien, $attribut, StringReplace(StringReplace(StringReplace($valeur,"","#"),@CR,""),@LF,""))
EndFunc
; ### copyLienAttribute($newLien,$oldLien,$attribut)>void :
; Ecrit la valeur de l'attribut $attribut de l'objet "Lien" $oldLien dans celui de l'objet "Lien" $newLien
Func copyLienAttribute(ByRef $newLien, $oldLien, $attribut)
	setLienAttribute($newLien, $attribut, getLienAttribute($oldLien, $attribut))
EndFunc

#EndRegion

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
