#Include-Once

#Region > Modlisation des objets "Module"

; ~~~ Prsentation de la structure ~~~

; L'objet "Module" est une reprsentation d'un module activable dans le nAiO; lui sont rattachs les attributs "privs" suivants :
;  - nom    [string-64]  : nom unique associ au module
;  - chemin [string-256] : chemin du fichier source du module, ou son excutable
;  - etat   [string-1]   : tat du module (N:nouveau, I:inactif, A:actif)
; La liste de modules est une reprsentation des modules enregistrs; lui est rattache la modlisation suivante :
;  - Array de taille n+1, o n reprsente le nombre de modules enregistrs
;    - [indice 0] : valeur gale au nombre de modules enregistrs, soit n
;    - [indice 1] : premier module (objet "Module")
;    - ...
;    - [indice n] : (n)ime module (objet "Module")
;  - Une liste vierge de modules est donc reprsente ainsi : Dim $array[1] = [0]



; ~~~ Constructeurs et fonctions de base ~~~

; ### newModule($nom,$chemin,$etat)>[Module] :
; Construit un objet "Module"  partir des donnes $nom, $chemin et $etat, et retourne sa valeur
; => $etat est optionnel
Func newModule($nom="", $chemin="", $etat="N")
	Local $module = DllStructCreate(getStructureModule())
	If $nom = "" Then Return ""
	setModuleNom($module, $nom)
	setModuleChemin($module, $chemin)
	setModuleEtat($module, $etat)
	Return $module
EndFunc
; ### copyModule($oldModule)>[Module] :
; Construit un objet "Module"  partir des donnes dupliques d'un autre objet "Module", et retourne la nouvelle valeur
Func copyModule($oldModule)
	Local $newModule = newModule(), $structure = getStructureModuleArray()
	For $i = 0 To getStructureModuleNumber()-1
		copyModuleAttribute($newModule, $oldModule, $structure[$i][1])
	Next
	Return $newModule
EndFunc
; ### deleteModule($module)>void :
; Detruit l'objet "Module" $module
Func deleteModule(ByRef $module)
	$module = 0
EndFunc
; ### isModule($module)>boolean :
; Retourne la valeur True si $module est bien un objet "Module", False sinon
Func isModule($module)
	Local $i = 0, $structure = getStructureModuleArray()
	If Not IsDllStruct($module) Then
		Return False
	EndIf
	For $i=0 To getStructureModuleNumber()-1
		Local $valeur = DllStructGetData($module, $structure[$i][1])
		If @error Then Return False
	Next
	Return True
EndFunc
; ### toStringModule($compte)>string :
; Retourne une chaine de caractres identifiant un objet "Module"
Func toStringModule($module)
	Return cond(isModuleEtatNouveau($module),"* ","")&getModuleNom($module)&cond(isModuleCompiled($module),""," [src]")
EndFunc
; ### newListeModules()>array :
; Cre une liste vierge de modules
Func newListeModules()
	Local $retour[1]
	setListeModulesLength($retour,0)
	Return $retour
EndFunc
; ### deleteListeModules($liste)>void :
; Ecrase $liste avec une liste vierge d'objets "Module" aprs avoir dtruit chaque objet "Module" prsent
Func deleteListeModules(ByRef $liste)
	If IsArray($liste) Then
		For $i = 0 To UBound($liste)-1
			$liste[$i] = 0
		Next
	EndIf
	$liste = newListeModules()
EndFunc



; ~~~ Fonctions de dbugage ~~~

; ### displayModule($module)>void :
; Affiche les informations de $module dans une fentre type MsgBox
Func displayModules($liste)
	For $i = 1 To getListeModulesLength($liste)
		displayModule(getModuleInListAt($liste,$i),$i&"/"&getListeModulesLength($liste))
	Next
EndFunc
; ### displayModules($liste)>void :
; Afficher les informations des lments de $liste dans des fentres ordonnes type MsgBox
Func displayModule($module, $title="")
	Local $fullTitle = "Module"
	If $title<>"" Then $fullTitle = $fullTitle & " ["&$title&"]"
	If isModule($module) Then
		Local $structure = getStructureModuleArray(), $msg = _
			"Module.toString: '" & toStringModule($module) & "'" & @CRLF & _
			"Struct Size: " & DllStructGetSize($module) & @CRLF & _
			"Struct pointer: " & DllStructGetPtr($module) & @CRLF & _
			"Data:"
		For $i = 0 To getStructureModuleNumber()-1
			$msg &= @CRLF & @TAB & $structure[$i][1]&": '"&getModuleAttribute($module, $structure[$i][1])&"'"
		Next
		MsgBox(0, $fullTitle, "{"&@CRLF&$msg&@CRLF&"}" )
	Else
		MsgBox(16, $fullTitle, "!!! Ce module n'est pas un objet conforme !!!" )
	EndIf
EndFunc



; ~~~ Fonctions Get/Set pour les attributs index, alias et avatar ~~~

; ### getModuleNom($module)>string :
; Lit l'attribut nom de $module, et retourne sa valeur
Func getModuleNom($module)
	Return getModuleAttribute($module, "nom")
EndFunc
; ### setModuleNom($module,$nom)>void :
; Ecrit la valeur $nom dans l'attribut nom de $module
Func setModuleNom(ByRef $module, $nom)
	setModuleAttribute($module, "nom", $nom)
EndFunc
; ### getModuleChemin($module)>string :
; Lit l'attribut chemin de $module, et retourne sa valeur
Func getModuleChemin($module)
	Return getModuleAttribute($module, "chemin")
EndFunc
; ### setModuleChemin($module,$chemin)>void :
; Ecrit la valeur $chemin dans l'attribut chemin de $module
Func setModuleChemin(ByRef $module, $chemin)
	setModuleAttribute($module, "chemin", $chemin)
EndFunc
; ### getModuleEtat($module)>string :
; Lit l'attribut etat de $module, et retourne sa valeur
Func getModuleEtat($module)
	Return getModuleAttribute($module, "etat")
EndFunc
; ### isModuleEtatNouveau($module)>boolean :
; Retourne la valeur True si l'attribut etat de $module est gal  "N", False sinon
Func isModuleEtatNouveau($module)
	Return (getModuleEtat($module)="N")
EndFunc
; ### isModuleEtatInactif($module)>boolean :
; Retourne la valeur True si l'attribut etat de $module est gal  "I", False sinon
Func isModuleEtatInactif($module)
	Return (getModuleEtat($module)="I")
EndFunc
; ### isModuleEtatActif($module)>boolean :
; Retourne la valeur True si l'attribut etat de $module est gal  "A", False sinon
Func isModuleEtatActif($module)
	Return (getModuleEtat($module)="A")
EndFunc
; ### setModuleEtat($module,$etat)>void :
; Ecrit la valeur $etat dans l'attribut etat de $module
Func setModuleEtat(ByRef $module, $etat)
	setModuleAttribute($module, "etat", $etat)
EndFunc
; ### setModuleEtatNouveau($module)>void :
; Ecrit la valeur "N" dans l'attribut etat de $module
Func setModuleEtatNouveau(ByRef $module)
	setModuleEtat($module, "N")
EndFunc
; ### setModuleEtatInactif($module)>void :
; Ecrit la valeur "I" dans l'attribut etat de $module
Func setModuleEtatInactif(ByRef $module)
	setModuleEtat($module, "I")
EndFunc
; ### setModuleEtatActif($module)>void :
; Ecrit la valeur "A" dans l'attribut etat de $module
Func setModuleEtatActif(ByRef $module)
	setModuleEtat($module, "A")
EndFunc
; ### isModuleCompiled($module)>boolean :
; Retourne la valeur True si le chemin du module $module est un fichier executable, False sinon
Func isModuleCompiled($module)
	Return StringRegExp(getModuleChemin($module),"\.exe$") = 1
EndFunc



; ~~~ Fonctions spcifique pour les modules ~~~

; ### isModuleTypeConnexion($module)>string :
; Retourne la valeur True si le module $module est un module de connexion, False sinon
; =>
Func isModuleTypeConnexion($module)
	Local $chemin = getModuleChemin($module)
	If FileExists($chemin) Then
		Local $propriete = FileGetProperty($chemin, "Type")
		If $propriete <> "" Then
			Return StringUpper($propriete) = "CONNEXION"
		Else
			SetError(1,0,True)
		EndIf
	Else
		SetError(1,0,False)
	EndIf
EndFunc



; ~~~ Fonctions de listing de modules ~~~

; ### addToListeModules($liste, $module)>void :
; Ajoute l'objet "Module" $module  la fin de la liste de modules $liste
; => Si un objet "Module" ayant la mme valeur pour l'attribut nom existe dj dans la liste, aucune action n'est effectue
Func addToListeModules(ByRef $liste, $module)
	Local $nom = getModuleNom($module)
	If Not isModule(getModuleInListFromNom($liste,$nom)) Then
		setListeModulesLength($liste,getListeModulesLength($liste)+1)
		setModuleInListAt($liste,getListeModulesLength($liste),$module)
	EndIf
EndFunc
; ### delInListeModulesAt($liste,$i)>[Module] :
; Efface l'objet "Module" prsent  l'indice $i de la liste de modules $liste, et retourne l'objet "Module" qui a t retir
; => Si l'indice $i n'est pas sens contenir un objet "Module" ($i<1 ou $i>n), la chaine vide "" est retourne
Func delInListeModulesAt(ByRef $liste, $i)
	Local $deleted = ""
	If $i>=1 And $i<=getListeModulesLength($liste) Then
		$deleted = getModuleInListAt($liste,$i)
		For $j = $i To getListeModulesLength($liste)-1
			setModuleInListAt($liste,$j,getModuleInListAt($liste,$j+1))
		Next
		setListeModulesLength($liste,getListeModulesLength($liste)-1)
	EndIf
	Return $deleted
EndFunc
; ### getModuleCheminInListFromNom($liste,$nom)>string :
; Retourne l'attribut chemin de l'objet "Module" prsent dans la liste de modules $liste ayant son attribut nom gal  $nom
; => Si aucun objet "Module" ne rpond au critre, la chaine vide "" est retourne
Func getModuleCheminInListFromNom($liste,$nom)
	Return getModuleChemin(getModuleInListFromNom($liste,$nom))
EndFunc
; ### getModuleInListFromNom($liste,$nom)>[Module] :
; Retourne l'objet "Module" prsent dans la liste de modules $liste ayant son attribut nom gal  $nom
; => Si aucun objet "Module" ne rpond au critre, la chaine vide "" est retourne
Func getModuleInListFromNom($liste,$nom)
	For $i = 1 To getListeModulesLength($liste)
		Local $module = getModuleInListAt($liste,$i)
		If isModule($module) And getModuleNom($module)=$nom Then Return $module
	Next
	Return ""
EndFunc
; ### getModuleInListFromChemin($liste,$chemin)>[Module] :
; Retourne l'objet "Module" prsent dans la liste de modules $liste ayant son attribut chemin gal  $chemin
; => Si aucun objet "Module" ne rpond au critre, la chaine vide "" est retourne
Func getModuleInListFromChemin($liste,$chemin)
	For $i = 1 To getListeModulesLength($liste)
		Local $module = getModuleInListAt($liste,$i)
		If isModule($module) And getModuleChemin($module)=$chemin Then Return $module
	Next
	Return ""
EndFunc
; ### getModuleInListFromToString($liste,$toString)>[Module] :
; Retourne l'objet "Module" prsent dans la liste de modules $liste ayant sa valeur d'affichage "ToString" gale  $toString
; => Si aucun objet "Module" ne rpond au critre, la chaine vide "" est retourne
Func getModuleInListFromToString($liste,$toString)
	For $i = 1 To getListeModulesLength($liste)
		Local $module = getModuleInListAt($liste,$i)
		If isModule($module) And toStringModule($module)=$toString Then Return $module
	Next
	Return ""
EndFunc
; ### setModuleCheminInListFromNom($liste,$nom,$chemin)>[Module] :
; Modifie l'objet "Module" prsent dans la liste de modules $liste ayant son attribut nom gal  $nom, en crivant $chemin dans son attribut chemin, et retourne l'lment avant modification
; => Si aucun objet "Module" ne rpond au critre, la chaine vide "" est retourne
Func setModuleCheminInListFromNom($liste,$nom,$chemin)
	For $i = 1 To getListeModulesLength($liste)
		Local $modified = getModuleInListAt($liste,$i)
		If isModule($modified) And getModuleNom($modified)=$nom Then
			setModuleInListAt($liste,$i,newModule($nom,$chemin))
			Return $modified
		EndIf
	Next
	Return ""
EndFunc
; ### setModuleInListFromNom($liste,$nom,$module)>[Module] :
; Modifie l'objet "Module" prsent dans la liste de modules $liste ayant son attribut nom gal  $nom, en le replaant par l'objet "Module" $module, et retourne l'lment avant modification
; => Si aucun objet "Module" ne rpond au critre, la chaine vide "" est retourne
Func setModuleInListFromNom($liste,$nom,$module)
	For $i = 1 To getListeModulesLength($liste)
		Local $modified = getModuleInListAt($liste,$i)
		If isModule($modified) And getModuleNom($modified)=$nom Then
			setModuleInListAt($liste,$i,$module)
			Return $modified
		EndIf
	Next
	Return ""
EndFunc
; ### getModuleInListAt($liste,$i)>[Module] :
; Retourne l'objet "Module" prsent  l'indice $i dans la liste de modules $liste
; => Si l'indice $i ne contient pas un objet "Module", la chaine vide "" est retourne
Func getModuleInListAt($liste,$i)
	If isModule($liste[$i]) Then
		Return $liste[$i]
	Else
		Return ""
	EndIf
EndFunc
; ### setModuleInListAt($liste,$i,$module)>void :
; Ecrit l'objet "Module" $module  l'indice $i de la liste de modules $liste
; => Si l'indice $i est invalide, aucune action n'est effectue
Func setModuleInListAt(ByRef $liste,$i,$module)
	If $i>0 And $i<=getListeModulesLength($liste) Then
		$liste[$i] = $module
	EndIf
EndFunc
; ### getListeModulesLength($liste)>int :
; Retourne le nombre d'objets "Module" prsent dans la liste de modules $liste
Func getListeModulesLength($liste)
	If Not IsArray($liste) Then Return 0
	Return $liste[0]
EndFunc
; ### setListeModulesLength($liste,$length)>void :
; Redimensionne  $length la taille de la liste de modules $liste
Func setListeModulesLength(ByRef $liste, $length)
	If $length>=0 Then
		$liste[0] = $length
		ReDim $liste[getListeModulesLength($liste)+1]
	EndIf
EndFunc



; ~~~ Fonctions pour mashall/unmarshall ~~~

; ### convertLigneToModule($ligne)>[Module] :
; Transforme la chaine de caractres $ligne en objet "Module" (unmarshall)
Func convertLigneToModule($ligne)
	Local $param = StringSplit($ligne,"")
	If $param[0] = getStructureModuleNumber() Then Return newModule($param[1],$param[2],$param[3])
	SetError(1)
EndFunc
; ### convertModuleToLigne($module)>string :
; Srialise l'objet "Module" $module en chaine de caractres (marshall)
Func convertModuleToLigne($module)
	Return getModuleNom($module)&""&getModuleChemin($module)&""&cond(isModuleEtatActif($module),"A","I")
EndFunc
; ### convertFichierToListeModules($chemin,$liste)>void :
; Lit les donnes du fichier de config $chemin, et enregistre la liste de modules correspondante dans $liste
; => Les donnes de $liste sont crases
Func convertFichierToListeModules($chemin, ByRef $liste, $dossier = "")
	$liste = newListeModules()
	Local $fichier = FileOpen($chemin, 0)
	If $dossier = "" And IsDeclared("nAiO_repertoire_modules") Then
		$dossier = $nAiO_repertoire_modules
	EndIf
	While $fichier <> -1
		Local $ligne = FileReadLine($fichier)
		If @error <> 0 Then ExitLoop
		Local $module = convertLigneToModule($ligne)
		If @error = 0 And FileExists(getModuleChemin($module)) Then addToListeModules($liste, $module)
	WEnd
	FileClose($fichier)
	If $dossier <> "" And FileExists($dossier) Then
		Local $attrib = FileGetAttrib($dossier)
		If @error Then
			Return
		ElseIf Not StringInStr($attrib, "D") Then
			Return
		EndIf
		Local $modules_AU3 = _FileListToArray($dossier,"*.au3",1)
		If IsArray($modules_AU3) Then
			For $i = 1 To $modules_AU3[0]
				If Not isModule(getModuleInListFromNom($liste,StringTrimRight($modules_AU3[$i],4))) Then
					addToListeModules($liste, newModule(StringTrimRight($modules_AU3[$i],4), $dossier&$modules_AU3[$i], "N"))
				EndIf
			Next
		EndIf
		Local $modules_EXE = _FileListToArray($dossier,"*.exe",1)
		If IsArray($modules_EXE) Then
			For $i = 1 To $modules_EXE[0]
				If Not isModule(getModuleInListFromNom($liste,StringTrimRight($modules_EXE[$i],4))) Then
					addToListeModules($liste, newModule(StringTrimRight($modules_EXE[$i],4), $dossier&$modules_EXE[$i], "N"))
				EndIf
			Next
		EndIf
	EndIf
EndFunc
; ### convertListeModulesToFichier($liste,$chemin)>void :
; Lit les donnes de la liste de modules $liste et enregistre les donnes correspondantes dans le fichier de config $chemin
; => Les donnes du fichier $chemin sont crases
Func convertListeModulesToFichier($liste, $chemin)
	Local $texte = ""
	For $i = 1 To getListeModulesLength($liste)
		If $texte <> "" Then $texte = $texte & @CRLF
		$texte = $texte & convertModuleToLigne(getModuleInListAt($liste,$i))
	Next
	Local $fichier = FileOpen($chemin, 10)
	If $fichier <> -1 Then FileWrite($fichier, $texte)
	FileClose($fichier)
EndFunc



; ~~~ Fonctions internes  la modlisation,  ne pas utiliser hors de ce fichier ~~~

; ### getStructureModuleArray()>matrix :
; Retourne une matrice contenant les attributs et leurs formats, pour structurer un objet "Module"
Func getStructureModuleArray()
	Local $structure[3][3] = [["char","nom",64], ["char","chemin",256], ["char","etat",1]]
	Return $structure
EndFunc
; ### getStructureModuleNumber()>int :
; Retourne le nombre d'attributs ncessaires pour structurer un objet "Module"
Func getStructureModuleNumber()
	Return UBound(getStructureModuleArray(),1)
EndFunc
; ### getStructureModule()>string :
; Retourne une chaine de caractres contenant les attributs et leurs formats, pour structurer un objet "Module"
Func getStructureModule()
	Local $i = 0, $string = "", $structure = getStructureModuleArray()
	For $i=0 To getStructureModuleNumber()-1
		If $string <> "" Then $string &= ";"
		$string &= $structure[$i][0] & " " & $structure[$i][1] & cond($structure[$i][2]<>"", "["&$structure[$i][2]&"]", "")
	Next
	Return $string
EndFunc
; ### getModuleAttribute($module,$attribut)>string :
; Retourne l'attribut $attribut de l'objet "Module" $module
Func getModuleAttribute($module, $attribut)
	Return DllStructGetData($module, $attribut)
EndFunc
; ### setModuleAttribute($module,$attribut,$valeur)>void :
; Ecrit la valeur $valeur dans l'attribut $attribut de l'objet "Module" $module
Func setModuleAttribute(ByRef $module, $attribut, $valeur)
	DllStructSetData($module, $attribut, $valeur)
EndFunc
; ### copyModuleAttribute($newModule,$oldModule,$attribut)>void :
; Ecrit la valeur de l'attribut $attribut de l'objet "Module" $oldModule dans celui de l'objet "Module" $newModule
Func copyModuleAttribute(ByRef $newModule, $oldModule, $attribut)
	setModuleAttribute($newModule, $attribut, getModuleAttribute($oldModule, $attribut))
EndFunc

#EndRegion

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
