#Include-Once

; Librairies nAiO
#Include "..\Encryptage Decryptage.au3"
#Include "..\Fonctions Avancees.au3"
#Include "Compte.au3"

Global Const $MULTI_LISTING_SEPARATOR = ";"

#Region > Modlisation des objets "Multi"

; ~~~ Prsentation de la structure ~~~

; L'objet "Multi" est une reprsentation d'un multiconnecteur de jeu Dofus; lui sont rattachs les attributs "privs" suivants :
;  - index   [int]        : indice permettant un accs rapide au multi. les valeurs de "index" doivent tre diffrents deux  deux pour chaque multi existant
;  - alias   [string-32]  : nom du multi facilitant la reconnaissance par l'utilisateur
;  - listing [string-256] : liste des index des objets "Compte" concerns
; La liste de multis est une reprsentation des multis enregistrs; lui est rattache la modlisation suivante :
;  - Array de taille n+1, o n reprsente le nombre de multis enregistrs
;    - [indice 0] : valeur gale au nombre de multis enregistrs, soit n
;    - [indice 1] : premier multi (objet "Multi")
;    - ...
;    - [indice n] : (n)ime multi (objet "Multi")
;  - Une liste vierge de multis est donc reprsente ainsi : Dim $array[1] = [0]

; ~~~ Constructeurs et fonctions de base ~~~

; ### newMulti($index,$alias,$listing)>[Multi] :
; Construit un objet "Multi"  partir des donnes $index, $alias et $listing, et retourne sa valeur
; => Tous les paramtres sont tous optionnels
Func newMulti($index="", $alias="", $listing="")
	Local $multi = DllStructCreate(getStructureMulti())
	If $index <> "" Then setMultiIndex($multi, $index)
	setMultiAlias($multi, $alias)
	setMultiListing($multi, $listing)
	Return $multi
EndFunc
; ### copyMulti($oldMulti)>[Multi] :
; Construit un objet "Multi"  partir des donnes dupliques d'un autre objet "Multi", et retourne la nouvelle valeur
Func copyMulti($oldMulti)
	Local $newMulti = newMulti(), $structure = getStructureMultiArray()
	For $i = 0 To getStructureMultiNumber()-1
		copyMultiAttribute($newMulti, $oldMulti, $structure[$i][1])
	Next
	Return $newMulti
EndFunc
; ### deleteMulti($multi)>void :
; Detruit l'objet "Multi" $multi
Func deleteMulti(ByRef $multi)
	$multi = 0
EndFunc
; ### isMulti($multi)>boolean :
; Retourne la valeur True si $multi est bien un objet "Multi", False sinon
Func isMulti($multi)
	Local $i = 0, $structure = getStructureMultiArray()
	If Not IsDllStruct($multi) Then
		Return False
	EndIf
	For $i=0 To getStructureMultiNumber()-1
		Local $valeur = DllStructGetData($multi, $structure[$i][1])
		If @error Then Return False
	Next
	Return True
EndFunc
; ### toStringMulti($multi)>string :
; Retourne une chaine de caractres identifiant un objet "Multi"
Func toStringMulti($multi)
	Return getMultiIndex($multi)&" - "&getMultiAlias($multi)
EndFunc
; ### newListeMultis()>array :
; Cre une liste vierge de multis
Func newListeMultis()
	Local $retour[1]
	setListeMultisLength($retour,0)
	Return $retour
EndFunc
; ### deleteListeMultis($liste)>void :
; Ecrase $liste avec une liste vierge d'objets "Multi" aprs avoir dtruit chaque objet "Multi" prsent
Func deleteListeMultis(ByRef $liste)
	If IsArray($liste) Then
		For $i = 0 To UBound($liste)-1
			$liste[$i] = 0
		Next
	EndIf
	$liste = newListeMultis()
EndFunc

; ~~~ Fonctions de dbugage ~~~

; ### displayMulti($multi)>void :
; Affiche les informations de $multi dans une fentre type MsgBox
Func displayMultis($liste)
	For $i = 1 To getListeMultisLength($liste)
		displayMulti(getMultiInListAt($liste,$i),$i&"/"&getListeMultisLength($liste))
	Next
EndFunc
; ### displayMultis($liste)>void :
; Afficher les informations des lments de $liste dans des fentres ordonnes type MsgBox
Func displayMulti($multi, $title="")
	Local $fullTitle = "Multi"
	If $title<>"" Then $fullTitle = $fullTitle & " ["&$title&"]"
	If isMulti($multi) Then
		Local $structure = getStructureMultiArray(), $msg = _
			"Multi.toString: '" & toStringMulti($multi) & "'" & @CRLF & _
			"Struct Size: " & DllStructGetSize($multi) & @CRLF & _
			"Struct pointer: " & DllStructGetPtr($multi) & @CRLF & _
			"Data:"
		For $i = 0 To getStructureMultiNumber()-1
			$msg &= @CRLF & @TAB & $structure[$i][1]&": '"&getMultiAttribute($multi, $structure[$i][1])&"'"
		Next
		MsgBox(0, $fullTitle, "{"&@CRLF&$msg&@CRLF&"}" )
	Else
		MsgBox(16, $fullTitle, "!!! Ce multi n'est pas un objet conforme !!!" )
	EndIf
EndFunc

; ~~~ Fonctions Get/Set pour les attributs index, alias et listing ~~~

; ### getMultiIndex($multi)>string :
; Lit l'attribut index de $multi
Func getMultiIndex($multi)
	Return getMultiAttribute($multi, "index")
EndFunc
; ### setMultiIndex($multi,$index)>void :
; Ecrit la valeur $index dans l'attribut index de $multi
Func setMultiIndex(ByRef $multi, $index)
	setMultiAttribute($multi, "index", $index)
EndFunc
; ### getMultiAlias($multi)>string :
; Lit l'attribut alias de $multi
Func getMultiAlias($multi)
	Return getMultiAttribute($multi, "alias")
EndFunc
; ### setMultiAlias($multi,$alias)>void :
; Ecrit la valeur $alias dans l'attribut alias de $multi
Func setMultiAlias(ByRef $multi, $alias)
	setMultiAttribute($multi, "alias", $alias)
EndFunc
; ### getMultiListing($multi)>string :
; Lit l'attribut listing de $multi
Func getMultiListing($multi)
	Return getMultiAttribute($multi, "listing")
EndFunc
; ### setMultiListing($multi,$listing)>void :
; Ecrit la valeur $listing (ou sa transcription en chaine de caractres, si $listing est au format Array) dans l'attribut listing de $multi
Func setMultiListing(ByRef $multi, $listing)
	Local $listingToSet = $listing
	If IsArray($listing) Then
		$listingToSet = ""
		For $i = 1 To $listing[0]
			$listingToSet = cond($listingToSet = "","",$listingToSet&$MULTI_LISTING_SEPARATOR)&$listing[$i]
		Next
	EndIf
	setMultiAttribute($multi, "listing", $listing)
EndFunc

; ~~~ Fonctions de manipulation de l'attribut listing ~~~

; ### getMultiListingSize($multi)>int :
; Retourne le nombre d'index prsents dans l'attribut listing de l'objet "Multi" $multi
Func getMultiListingSize($multi)
	Local $listingAsArray = getMultiListingAsArray($multi)
	Return $listingAsArray[0]
EndFunc
; ### getMultiListingAsArray($multi)>[int] :
; Retourne l'attribut listing de l'objet "Multi" $multi, sous forme d'une liste, l'indice 0 reprsente la taille de la liste
Func getMultiListingAsArray($multi)
	Local $listing = getMultiListing($multi), $resultat[1] = [0]
	If $listing = "" Then
		Return $resultat
	ElseIf Not StringInStr($listing, $MULTI_LISTING_SEPARATOR) Then
		ReDim $resultat[2]
		$resultat[0] = 1
		$resultat[1] = $listing
	Else
		$resultat = StringSplit($listing,$MULTI_LISTING_SEPARATOR)
	EndIf
	Return $resultat
EndFunc
; ### isCompteInMultiListing($multi,$compte)>boolean :
; Retourne True si l'index de l'objet "Compte" $compte est prsent dans l'attribut listing de l'objet "Multi" $multi ($multi peut aussi reprsenter le listing lui-mme)
Func isCompteInMultiListing($multi,$compte)
	Local $m = $MULTI_LISTING_SEPARATOR
	If isCompte($compte) Then
		Local $index = getCompteIndex($compte), $listing = String($multi)
		If isMulti($multi) Then $listing = getMultiListing($multi)
		Return (StringInStr($m&$listing&$m,$m&Number($index)&$m)<>0)
	EndIf
	Return False
EndFunc
; ### addCompteToMultiListing(ByRef $multi, $compte)>void :
; Ajoute l'index de l'objet "Compte" $compte au listing de l'objet "Multi" $multi
Func addCompteToMultiListing(ByRef $multi, $compte)
	If Not isCompteInMultiListing($multi,$compte) Then
		Local $listing = getMultiListing($multi), $index = getCompteIndex($compte)
		$listing = cond($listing="","",$listing&$MULTI_LISTING_SEPARATOR)&$index
		setMultiListing($multi, $listing)
	EndIf
EndFunc
; ### swapComptesInMultiListing(ByRef $multi, $compte1, $compte2, $listComptes)>void :
; Echange les places des index des objets "Compte" $compte1 et $compte2 dans le listing de l'objet "Multi" $multi
; => si $compte1 et $compte2 sont des comptes, $listeComptes est inutile
; => sinon, $compte1 et compte2 doivent correspondre  des chaines de caractres, et $listeComptes une liste d'objets "Compte"
Func swapComptesInMultiListing(ByRef $multi, $compte1, $compte2, $listeComptes = "")
	Local $ok = False, $object1 = $compte1, $object2 = $compte2
	If isCompte($compte1) And isCompte($compte2) Then
		$ok = True
	ElseIf IsArray($listeComptes) Then
		$object1 = getCompteInListFromIndex($listeComptes,$compte1)
		$object2 = getCompteInListFromIndex($listeComptes,$compte2)
		$ok = (isCompte($object1) And isCompte($object2))
	EndIf
	If $ok And isCompteInMultiListing($multi,$compte1) And isCompteInMultiListing($multi,$compte2) Then
		Local _
			$index1 = getCompteIndex($object1), _
			$index2 = getCompteIndex($object2), _
			$listingAsArray = getMultiListingAsArray($multi)
		For $i = 1 To $listingAsArray[0]
			If $listingAsArray[$i] = $index1 Then
				$listingAsArray[$i] = $index2
			ElseIf $listingAsArray[$i] = $index2 Then
				$listingAsArray[$i] = $index1
			EndIf
		Next
		setMultiListing($multi, $listingAsArray)
	EndIf
EndFunc
; ### delCompteFromMultiListing(ByRef $multi, $compte)>void :
; Retire l'index de l'objet "Compte" $compte du listing de l'objet "Multi" $multi
Func delCompteFromMultiListing(ByRef $multi, $compte)
	If isCompteInMultiListing($multi,$compte) Then
		Local $listing = "", $index = getCompteIndex($compte), $listingAsArray = getMultiListingAsArray($multi)
		For $i = 1 To $listingAsArray[0]
			If $listingAsArray[$i] <> $index Then $listing = $listing & $MULTI_LISTING_SEPARATOR & $listingAsArray[$i]
		Next
		setMultiListing($multi, StringTrimLeft($listing,StringLen($MULTI_LISTING_SEPARATOR)))
	EndIf
EndFunc
; ### checkMultiListingWithListeComptes($listing, $liste)>boolean :
; Retourne True si les index de $listing sont tous prsents dans la liste d'objets "Compte" $liste, False sinon
Func checkMultiListingWithListeComptes($listing, $liste)
	If $listing = "" Then Return True
	Local $listingAsArray = getMultiListingAsArray($listing)
	For $i = 1 To $listingAsArray[0]
		If Not isCompte(getCompteInListFromIndex($liste,$listingAsArray[$i])) Then Return False
	Next
	Return True
EndFunc
; ### cleanMultiListingWithListeComptes($listing, $liste)>string :
; Modifie le listing $listing en fonction de la liste de comptes $liste en retirant les index du listing qui ne sont pas prsents dans $liste, et retourne la nouvelle valeur
Func cleanMultiListingWithListeComptes($listing, $liste)
	If $listing = "" Then Return $listing
	Local $split = StringSplit($listing,$MULTI_LISTING_SEPARATOR), $chaine = ""
	For $i = 1 To $split[0]
		If isCompte(getCompteInListFromIndex($liste,$split[$i])) Then $chaine = $chaine & $MULTI_LISTING_SEPARATOR & $split[$i]
	Next
	Return StringTrimLeft($chaine, StringLen($MULTI_LISTING_SEPARATOR))
EndFunc
; ### getCompteIndexInMultiListingAt($multi,$i)>string :
; Retourne l'index du compte situ en position $i dans le listing de l'objet "Multi", si l'index est incorrect la chane vide est retourne
Func getCompteIndexInMultiListingAt($multi,$i)
	Local $listing = String($multi)
	If isMulti($multi) Then $listing = getMultiListing($multi)
	If $listing = "" Then Return ""
	Local $split = StringSplit($listing,$MULTI_LISTING_SEPARATOR)
	If IsArray($split) And $i > 0 And $i <= $split[0] Then Return $split[$i]
	Return ""
EndFunc

; ~~~ Fonctions de listing de multis ~~~

; ### addToListeMultis($liste, $multi)>void :
; Ajoute l'objet "Multi" $multi  la fin de la liste de multis $liste
; => La valeur  l'indice 0 est incrmente automatiquement si un objet "Multi" ayant la mme valeur index existe dj dans la liste
Func addToListeMultis(ByRef $liste, $multi)
	Local $indexMulti = getMultiIndex($multi)
	If isMulti(getMultiInListFromIndex($liste,$indexMulti)) Then
		setMultiIndex($multi, $indexMulti+1)
		addToListeMultis($liste, $multi)
	Else
		setListeMultisLength($liste,getListeMultisLength($liste)+1)
		setMultiInListAt($liste,getListeMultisLength($liste),$multi)
	EndIf
EndFunc
; ### delInListeMultisAt($liste,$i)>[Multi] :
; Efface l'objet "Multi" prsent  l'indice $i de la liste de multis $liste, et retourne l'objet "Multi" qui a t retir
; => Si l'indice $i n'est pas sens contenir un objet "Multi" ($i<1 ou $i>n), la chaine vide "" est retourne
Func delInListeMultisAt(ByRef $liste, $i)
	Local $deleted = ""
	If $i>=1 And $i<=getListeMultisLength($liste) Then
		$deleted = getMultiInListAt($liste,$i)
		For $j = $i To getListeMultisLength($liste)-1
			setMultiInListAt($liste,$j,getMultiInListAt($liste,$j+1))
		Next
		setListeMultisLength($liste,getListeMultisLength($liste)-1)
	EndIf
	Return $deleted
EndFunc
; ### delInListeMultisFromToString($liste,$toStringMulti)>[Multi] :
; Efface l'objet "Multi" ayant $toStringMulti comme identifiant, de la liste de multis $liste, et retourne l'objet "Multi" qui a t retir
; => Si aucun objet "Multi" ne correspond au critre, la chaine vide "" est retourne
Func delInListeMultisFromToString(ByRef $liste, $toStringMulti)
	For $i = 1 To getListeMultisLength($liste)
		If toStringMulti(getMultiInListAt($liste,$i)) = $toStringMulti Then ExitLoop
	Next
	If $i <= getListeMultisLength($liste) Then Return delInListeMultisAt($liste, $i)
	Return ""
EndFunc
; ### moveInListeMultisFromToString($liste, $toStringMulti1, $toStringMulti2)>void :
; Retire l'objet "Multi" ayant $toStringMulti1 comme identifiant, de la liste de multis $liste, et dplace les objets "Multi"
; dans la liste afin que l'objet "Multi" retir puisse tre  la place de celui ayant $toStringMulti2 comme identifiant
; => Si aucun objet "Multi" ne correspond au critre $toStringMulti1 ou $toStringMulti2, aucune action n'est effectue
Func moveInListeMultisFromToString(ByRef $liste, $toStringMulti1, $toStringMulti2)
	For $i = 1 To getListeMultisLength($liste)
		If toStringMulti(getMultiInListAt($liste,$i)) = $toStringMulti1 Then ExitLoop
	Next
	For $j = 1 To getListeMultisLength($liste)
		If toStringMulti(getMultiInListAt($liste,$j)) = $toStringMulti2 Then ExitLoop
	Next
	If $i <> $j And $i <= getListeMultisLength($liste) And $j <= getListeMultisLength($liste) Then
		Local $multiTemp = getMultiInListAt($liste,$i), $step = cond($i<$j,1,-1)
		For $k = $i To $j-$step Step $step
			setMultiInListAt($liste,$k,getMultiInListAt($liste,$k+$step))
		Next
		setMultiInListAt($liste,$j,$multiTemp)
	EndIf
EndFunc
; ### getMultiInListFromAlias($liste,$alias)>[Multi] :
; Retourne l'objet "Multi" prsent dans la liste de multis $liste ayant son attribut alias gal  $alias
; => Si aucun objet "Multi" ne rpond au critre, la chaine vide "" est retourne
Func getMultiInListFromAlias($liste,$alias)
	For $i = 1 To getListeMultisLength($liste)
		If isMulti(getMultiInListAt($liste,$i)) And getMultiAlias(getMultiInListAt($liste,$i))=$alias Then Return getMultiInListAt($liste,$i)
	Next
	Return ""
EndFunc
; ### getMultiInListFromIndex($liste,$index)>[Multi] :
; Retourne l'objet "Multi" prsent dans la liste de multis $liste ayant son attribut index gal  $index
; => Si aucun objet "Multi" ne rpond au critre, la chaine vide "" est retourne
Func getMultiInListFromIndex($liste,$index)
	For $i = 1 To getListeMultisLength($liste)
		If isMulti(getMultiInListAt($liste,$i)) And getMultiIndex(getMultiInListAt($liste,$i))=$index Then Return getMultiInListAt($liste,$i)
	Next
	Return ""
EndFunc
; ### getMultiInListFromToString($liste,$toString)>[Multi] :
; Retourner l'objet "Multi" prsent dans la liste de multis $liste ayant $toStringMulti comme identifiant
; => Si aucun objet "Multi" ne rpond au critre, la chaine vide "" est retourne
Func getMultiInListFromToString($liste,$toStringMulti)
	For $i = 1 To getListeMultisLength($liste)
		If isMulti(getMultiInListAt($liste,$i)) And toStringMulti(getMultiInListAt($liste,$i))=$toStringMulti Then Return getMultiInListAt($liste,$i)
	Next
	Return ""
EndFunc
; ### setMultiInListFromToString($liste,$toStringMulti,$alias,$listing)>[Multi] :
; Modifie les informations de l'objet "Multi" prsent dans la liste de multis $liste ayant $toStringMulti comme identifiant
; par les valeurs $alias et $listing, et retourne la valeur de l'objet "Multi" avant modification
; => Si aucun objet "Multi" ne rpond au critre, la chaine vide "" est retourne
Func setMultiInListFromToString(ByRef $liste, $toStringMulti, $alias,$listing)
	Local $modified = ""
	For $i = 1 To getListeMultisLength($liste)
		If isMulti(getMultiInListAt($liste,$i)) And toStringMulti(getMultiInListAt($liste,$i))=$toStringMulti Then
			Local $multi = getMultiInListAt($liste,$i)
			$modified = copyMulti($multi)
			setMultiAlias($multi,$alias)
			setMultiListing($multi,$listing)
		EndIf
	Next
	Return $modified
EndFunc
; ### getMultiInListAt($liste,$i)>[Multi] :
; Retourne l'objet "Multi" prsent  l'indice $i dans la liste de multis $liste
; => Si l'indice $i ne contient pas un objet "Multi", la chaine vide "" est retourne
Func getMultiInListAt($liste,$i)
	If isMulti($liste[$i]) Then
		Return $liste[$i]
	Else
		Return ""
	EndIf
EndFunc
; ### setMultiInListAt($liste,$i,$multi)>void :
; Ecrit l'objet "Multi" $multi  l'indice $i de la liste de multis $liste
; => Si l'indice $i est invalide, aucune action n'est effectue
Func setMultiInListAt(ByRef $liste,$i,$multi)
	If $i>0 And $i<=getListeMultisLength($liste) Then
		$liste[$i] = $multi
	EndIf
EndFunc
; ### getListeMultisLength($liste)>int :
; Retourne le nombre d'objets "Multi" prsent dans la liste de multis $liste
Func getListeMultisLength($liste)
	Return $liste[0]
EndFunc
; ### setListeMultisLength($liste,$length)>void :
; Redimensionne  $length la taille de la liste de multis $liste
Func setListeMultisLength(ByRef $liste, $length)
	If $length>=0 Then
		$liste[0] = $length
		ReDim $liste[getListeMultisLength($liste)+1]
	EndIf
EndFunc

; ~~~ Fonctions pour mashall/unmarshall ~~~

; ### convertLigneToMulti($ligne)>[Multi] :
; Transforme la chaine de caractres $ligne en objet "Multi" (unmarshall)
Func convertLigneToMulti($ligne)
	Local $param = StringSplit($ligne,"")
	If $param[0] = getStructureMultiNumber() Then Return newMulti($param[1],$param[2],StringTrimLeft(_HexToString($param[3]),StringLen($MULTI_LISTING_SEPARATOR)))
	SetError(1)
EndFunc
; ### convertMultiToLigne($multi)>string :
; Srialise l'objet "Multi" $multi en chaine de caractres (marshall)
Func convertMultiToLigne($multi)
	Return getMultiIndex($multi)&""&getMultiAlias($multi)&""&_StringToHex($MULTI_LISTING_SEPARATOR&getMultiListing($multi))
EndFunc
; ### convertFichierToListeMultis($chemin,$liste)>void :
; Lit les donnes du fichier de config $chemin, et enregistre la liste de multis correspondante dans $liste
; => Les donnes de $liste sont crases
Func convertFichierToListeMultis($chemin, ByRef $liste)
	$liste = newListeMultis()
	Local $fichier = FileOpen($chemin, 0)
	While $fichier <> -1
		Local $ligne = FileReadLine($fichier)
		If @error <> 0 Then ExitLoop
		Local $multi = convertLigneToMulti($ligne)
		If @error = 0 Then addToListeMultis($liste, $multi)
	WEnd
	FileClose($fichier)
EndFunc
; ### convertListeMultisToFichier($liste,$chemin)>void :
; Lit les donnes de la liste de multis $liste et enregistre les donnes correspondantes dans le fichier de config $chemin
; => Les donnes du fichier $chemin sont crases
Func convertListeMultisToFichier($liste, $chemin)
	Local $texte = ""
	For $i = 1 To getListeMultisLength($liste)
		If $texte <> "" Then $texte = $texte & @CRLF
		$texte = $texte & convertMultiToLigne(getMultiInListAt($liste,$i))
	Next
	Local $fichier = FileOpen($chemin, 10)
	If $fichier <> -1 Then FileWrite($fichier, $texte)
	FileClose($fichier)
EndFunc

; ~~~ Fonctions internes  la modlisation,  ne pas utiliser hors de ce fichier ~~~

; ### getStructureMultiArray()>matrix :
; Retourne une matrice contenant les attributs et leurs formats, pour structurer un objet "Multi"
Func getStructureMultiArray()
	Local $structure[3][3] = [["int","index",""], ["char","alias",32], ["char","listing",256]]
	Return $structure
EndFunc
; ### getStructureMultiNumber()>int :
; Retourne le nombre d'attributs ncessaires pour structurer un objet "Multi"
Func getStructureMultiNumber()
	Return UBound(getStructureMultiArray(),1)
EndFunc
; ### getStructureMulti()>string :
; Retourne une chaine de caractres contenant les attributs et leurs formats, pour structurer un objet "Multi"
Func getStructureMulti()
	Local $i = 0, $string = "", $structure = getStructureMultiArray()
	For $i=0 To getStructureMultiNumber()-1
		If $string <> "" Then $string &= ";"
		$string &= $structure[$i][0] & " " & $structure[$i][1] & cond($structure[$i][2]<>"", "["&$structure[$i][2]&"]", "")
	Next
	Return $string
EndFunc
; ### getMultiAttribute($multi,$attribut)>string :
; Retourne l'attribut $attribut de l'objet "Multi" $multi
Func getMultiAttribute($multi, $attribut)
	Return DllStructGetData($multi, $attribut)
EndFunc
; ### setMultiAttribute($multi,$attribut,$valeur)>void :
; Ecrit la valeur $valeur dans l'attribut $attribut de l'objet "Multi" $multi
Func setMultiAttribute(ByRef $multi, $attribut, $valeur)
	DllStructSetData($multi, $attribut, StringReplace($valeur,"","#"))
EndFunc
; ### copyMultiAttribute($newMulti,$oldMulti,$attribut)>void :
; Ecrit la valeur de l'attribut $attribut de l'objet "Multi" $oldMulti dans celui de l'objet "Multi" $newMulti
Func copyMultiAttribute(ByRef $newMulti, $oldMulti, $attribut)
	setMultiAttribute($newMulti, $attribut, getMultiAttribute($oldMulti, $attribut))
EndFunc

#EndRegion