#Include-Once

#Region > Modlisation des objets "Parametre"

; ~~~ Prsentation de la structure ~~~

; L'objet "Parametre" est une reprsentation d'un parametre de configuration du nAiO; lui sont rattachs les attributs "privs" suivants :
;  - clef [string-64] : identifiant clef du parametre dans le fichier de config
;  - valeur [string-128] : valeur associe au parametre
; La liste de parametres est une reprsentation des parametres enregistrs; lui est rattache la modlisation suivante :
;  - Array de taille n+1, o n reprsente le nombre de parametres enregistrs
;    - [indice 0] : valeur gale au nombre de parametres enregistrs, soit n
;    - [indice 1] : premier parametre (objet "Parametre")
;    - ...
;    - [indice n] : (n)ime parametre (objet "Parametre")
;  - Une liste vierge de parametres est donc reprsente ainsi : Dim $array[1] = [0]



; ~~~ Constructeurs et fonctions de base ~~~

; ### newParametre($clef,$valeur)>[Parametre] :
; Construit un objet "Parametre"  partir des donnes $clef et $valeur, et retourne sa valeur
; => $valeur est optionnel
Func newParametre($clef="", $valeur="")
	Local $parametre = DllStructCreate(getStructureParametre())
	If $clef = "" Then Return ""
	setParametreClef($parametre, $clef)
	setParametreValeur($parametre, $valeur)
	Return $parametre
EndFunc
; ### copyParametre($oldParametre)>[Parametre] :
; Construit un objet "Parametre"  partir des donnes dupliques d'un autre objet "Parametre", et retourne la nouvelle valeur
Func copyParametre($oldParametre)
	Local $newParametre = newParametre(), $structure = getStructureParametreArray()
	For $i = 0 To getStructureParametreNumber()-1
		copyParametreAttribute($newParametre, $oldParametre, $structure[$i][1])
	Next
	Return $newParametre
EndFunc
; ### deleteParametre($parametre)>void :
; Detruit l'objet "Parametre" $parametre
Func deleteParametre(ByRef $parametre)
	$parametre = 0
EndFunc
; ### isParametre($parametre)>boolean :
; Retourne la valeur True si $parametre est bien un objet "Parametre", False sinon
Func isParametre($parametre)
	Local $i = 0, $structure = getStructureParametreArray()
	If Not IsDllStruct($parametre) Then
		Return False
	EndIf
	For $i=0 To getStructureParametreNumber()-1
		Local $valeur = DllStructGetData($parametre, $structure[$i][1])
		If @error Then Return False
	Next
	Return True
EndFunc
; ### toStringParametre($parametre)>string :
; Retourne une chaine de caractres identifiant un objet "Parametre"
Func toStringParametre($parametre)
	Return getParametreClef($parametre) & cond(getParametreValeur($parametre)="",""," [" & getParametreClef($parametre) & "]")
EndFunc
; ### newListeParametres()>array :
; Cre une liste vierge de parametres
Func newListeParametres()
	Local $retour[1]
	setListeParametresLength($retour,0)
	Return $retour
EndFunc
; ### deleteListeParametres($liste)>void :
; Ecrase $liste avec une liste vierge d'objets "Parametre" aprs avoir dtruit chaque objet "Parametre" prsent
Func deleteListeParametres(ByRef $liste)
	If IsArray($liste) Then
		For $i = 0 To UBound($liste)-1
			$liste[$i] = 0
		Next
	EndIf
	$liste = newListeParametres()
EndFunc



; ~~~ Fonctions de dbugage ~~~

; ### displayParametre($parametre)>void :
; Affiche les informations de $parametre dans une fentre type MsgBox
Func displayParametres($liste)
	For $i = 1 To getListeParametresLength($liste)
		displayParametre(getParametreInListAt($liste,$i),$i&"/"&getListeParametresLength($liste))
	Next
EndFunc
; ### displayParametres($liste)>void :
; Afficher les informations des lments de $liste dans des fentres ordonnes type MsgBox
Func displayParametre($parametre, $title="")
	Local $fullTitle = "Parametre"
	If $title<>"" Then $fullTitle = $fullTitle & " ["&$title&"]"
	If isParametre($parametre) Then
		Local $structure = getStructureParametreArray(), $msg = _
			"Parametre.toString: '" & toStringParametre($parametre) & "'" & @CRLF & _
			"Struct Size: " & DllStructGetSize($parametre) & @CRLF & _
			"Struct pointer: " & DllStructGetPtr($parametre) & @CRLF & _
			"Data:"
		For $i = 0 To getStructureParametreNumber()-1
			$msg &= @CRLF & @TAB & $structure[$i][1]&": '"&getParametreAttribute($parametre, $structure[$i][1])&"'"
		Next
		MsgBox(0, $fullTitle, "{"&@CRLF&$msg&@CRLF&"}" )
	Else
		MsgBox(16, $fullTitle, "!!! Ce parametre n'est pas un objet conforme !!!" )
	EndIf
EndFunc



; ~~~ Fonctions Get/Set pour les attributs index, alias et avatar ~~~

; ### getParametreClef($parametre)>string :
; Lit l'attribut clef de $parametre, et retourne sa valeur
Func getParametreClef($parametre)
	Return ""&getParametreAttribute($parametre, "clef")
EndFunc
; ### setParametreClef($parametre,$clef)>void :
; Ecrit la valeur $clef dans l'attribut clef de $parametre
Func setParametreClef(ByRef $parametre, $clef)
	setParametreAttribute($parametre, "clef", $clef)
EndFunc
; ### getParametreValeur($parametre)>string :
; Lit l'attribut valeur de $parametre, et retourne sa valeur
Func getParametreValeur($parametre)
	Return ""&getParametreAttribute($parametre, "valeur")
EndFunc
; ### setParametreValeur($parametre,$valeur)>void :
; Ecrit la valeur $valeur dans l'attribut valeur de $parametre
Func setParametreValeur(ByRef $parametre, $valeur)
	setParametreAttribute($parametre, "valeur", $valeur)
EndFunc



; ~~~ Fonctions de listing de parametres ~~~

; ### addToListeParametres($liste, $parametre)>void :
; Ajoute l'objet "Parametre" $parametre  la fin de la liste de parametres $liste
; => Si un objet "Parametre" ayant la mme valeur pour l'attribut clef existe dj dans la liste, aucune action n'est effectue
Func addToListeParametres(ByRef $liste, $parametre)
	Local $clef = getParametreClef($parametre)
	If Not isParametre(getParametreInListFromClef($liste,$clef)) Then
		setListeParametresLength($liste,getListeParametresLength($liste)+1)
		setParametreInListAt($liste,getListeParametresLength($liste),$parametre)
	EndIf
EndFunc
; ### delInListeParametresAt($liste,$i)>[Parametre] :
; Efface l'objet "Parametre" prsent  l'indice $i de la liste de parametres $liste, et retourne l'objet "Parametre" qui a t retir
; => Si l'indice $i n'est pas sens contenir un objet "Parametre" ($i<1 ou $i>n), la chaine vide "" est retourne
Func delInListeParametresAt(ByRef $liste, $i)
	Local $deleted = ""
	If $i>=1 And $i<=getListeParametresLength($liste) Then
		$deleted = getParametreInListAt($liste,$i)
		For $j = $i To getListeParametresLength($liste)-1
			setParametreInListAt($liste,$j,getParametreInListAt($liste,$j+1))
		Next
		setListeParametresLength($liste,getListeParametresLength($liste)-1)
	EndIf
	Return $deleted
EndFunc
; ### getParametreValeurInListFromClef($liste,$clef)>string :
; Retourne l'attribut valeur de l'objet "Parametre" prsent dans la liste de parametres $liste ayant son attribut clef gal  $clef
; => Si aucun objet "Parametre" ne rpond au critre, la chaine vide "" est retourne
Func getParametreValeurInListFromClef($liste,$clef)
	Return getParametreValeur(getParametreInListFromClef($liste,$clef))
EndFunc
; ### getParametreInListFromClef($liste,$clef)>[Parametre] :
; Retourne l'objet "Parametre" prsent dans la liste de parametres $liste ayant son attribut clef gal  $clef
; => Si aucun objet "Parametre" ne rpond au critre, la chaine vide "" est retourne
Func getParametreInListFromClef($liste,$clef)
	For $i = 1 To getListeParametresLength($liste)
		Local $parametre = getParametreInListAt($liste,$i)
		If isParametre($parametre) And getParametreClef($parametre)=$clef Then Return $parametre
	Next
	Return ""
EndFunc
; ### setParametreValeurInListFromClef($liste,$clef,$valeur,$force)>[Parametre] :
; Modifie l'objet "Parametre" prsent dans la liste de parametres $liste ayant son attribut clef gal  $clef, en crivant $valeur dans son attribut valeur, et retourne l'lment avant modification
; => Si aucun objet "Parametre" ne rpond au critre et que le boolen $force est  False, la chaine vide "" est retourne; si $force est  True, l'objet "Parametre" est ajout  la liste des parametres $liste
Func setParametreValeurInListFromClef(ByRef $liste,$clef,$valeur,$force=False)
	For $i = 1 To getListeParametresLength($liste)
		Local $modified = getParametreInListAt($liste,$i)
		If isParametre($modified) And getParametreClef($modified)=$clef Then
			setParametreInListAt($liste,$i,newParametre($clef,$valeur))
			Return $modified
		EndIf
	Next
	addToListeParametres($liste, newParametre($clef, $valeur))
	Return getParametreInListFromClef($liste,$clef)
EndFunc
; ### getParametreInListAt($liste,$i)>[Parametre] :
; Retourne l'objet "Parametre" prsent  l'indice $i dans la liste de parametres $liste
; => Si l'indice $i ne contient pas un objet "Parametre", la chaine vide "" est retourne
Func getParametreInListAt($liste,$i)
	If isParametre($liste[$i]) Then
		Return $liste[$i]
	Else
		Return ""
	EndIf
EndFunc
; ### setParametreInListAt($liste,$i,$parametre)>void :
; Ecrit l'objet "Parametre" $parametre  l'indice $i de la liste de parametres $liste
; => Si l'indice $i est invalide, aucune action n'est effectue
Func setParametreInListAt(ByRef $liste,$i,$parametre)
	If $i>0 And $i<=getListeParametresLength($liste) Then
		$liste[$i] = $parametre
	EndIf
EndFunc
; ### getListeParametresLength($liste)>int :
; Retourne le nombre d'objets "Parametre" prsent dans la liste de parametres $liste
Func getListeParametresLength($liste)
	Return $liste[0]
EndFunc
; ### setListeParametresLength($liste,$length)>void :
; Redimensionne  $length la taille de la liste de parametres $liste
Func setListeParametresLength(ByRef $liste, $length)
	If $length>=0 Then
		$liste[0] = $length
		ReDim $liste[getListeParametresLength($liste)+1]
	EndIf
EndFunc



; ~~~ Fonctions pour mashall/unmarshall ~~~

; ### convertFichierToListeParametres($chemin,$liste)>void :
; Lit les donnes du fichier de config $chemin, et enregistre la liste de parametres correspondante dans $liste
; => Les donnes de $liste sont crases
Func convertFichierToListeParametres($chemin, ByRef $liste)
	$liste = newListeParametres()
	Local $tab = IniReadSection($chemin, "ZaDS-nAiO")
	If @error = 0 Then
		For $i = 1 To $tab[0][0]
			Local $parametre = newParametre($tab[$i][0],$tab[$i][1])
			If @error = 0 Then addToListeParametres($liste, $parametre)
		Next
	EndIf
EndFunc
; ### convertListeParametresToFichier($liste,$chemin)>void :
; Lit les donnes de la liste de parametres $liste et enregistre les donnes correspondantes dans le fichier de config $chemin
; => Les donnes du fichier $chemin sont crases
Func convertListeParametresToFichier($liste, $chemin)
	For $i = 1 To getListeParametresLength($liste)
		Local $parametre = getParametreInListAt($liste,$i)
		IniWrite($chemin, "ZaDS-nAiO", getParametreClef($parametre), getParametreValeur($parametre))
	Next
EndFunc



; ~~~ Fonctions internes  la modlisation,  ne pas utiliser hors de ce fichier ~~~

; ### getStructureParametreArray()>matrix :
; Retourne une matrice contenant les attributs et leurs formats, pour structurer un objet "Parametre"
Func getStructureParametreArray()
	Local $structure[2][3] = [["char","clef",64], ["char","valeur",256]]
	Return $structure
EndFunc
; ### getStructureParametreNumber()>int :
; Retourne le nombre d'attributs ncessaires pour structurer un objet "Parametre"
Func getStructureParametreNumber()
	Return UBound(getStructureParametreArray(),1)
EndFunc
; ### getStructureParametre()>string :
; Retourne une chaine de caractres contenant les attributs et leurs formats, pour structurer un objet "Parametre"
Func getStructureParametre()
	Local $i = 0, $string = "", $structure = getStructureParametreArray()
	For $i=0 To getStructureParametreNumber()-1
		If $string <> "" Then $string &= ";"
		$string &= $structure[$i][0] & " " & $structure[$i][1] & cond($structure[$i][2]<>"", "["&$structure[$i][2]&"]", "")
	Next
	Return $string
EndFunc
; ### getParametreAttribute($parametre,$attribut)>string :
; Retourne l'attribut $attribut de l'objet "Parametre" $parametre
Func getParametreAttribute($parametre, $attribut)
	Return DllStructGetData($parametre, $attribut)
EndFunc
; ### setParametreAttribute($parametre,$attribut,$valeur)>void :
; Ecrit la valeur $valeur dans l'attribut $attribut de l'objet "Parametre" $parametre
Func setParametreAttribute(ByRef $parametre, $attribut, $valeur)
	DllStructSetData($parametre, $attribut, $valeur)
EndFunc
; ### copyParametreAttribute($newParametre,$oldParametre,$attribut)>void :
; Ecrit la valeur de l'attribut $attribut de l'objet "Parametre" $oldParametre dans celui de l'objet "Parametre" $newParametre
Func copyParametreAttribute(ByRef $newParametre, $oldParametre, $attribut)
	setParametreAttribute($newParametre, $attribut, getParametreAttribute($oldParametre, $attribut))
EndFunc

#EndRegion

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Func convertTimeOutToInt($timeout)
	Local $coef = 0
	If     StringRegExp($timeOut,"^[1-9]\d* ans?$")     Then
		$coef = 535680 ; 535680 minutes = 1 an
	ElseIf StringRegExp($timeOut,"^[1-9]\d* mois$")     Then
		$coef = 44640 ; 44640 minutes = 1 mois
	ElseIf StringRegExp($timeOut,"^[1-9]\d* jours?$")   Then
		$coef = 1440 ; 1440 minutes = 1 jour
	ElseIf StringRegExp($timeOut,"^[1-9]\d* heures?$")  Then
		$coef = 60 ; 60 minutes = 1 heure
	ElseIf StringRegExp($timeOut,"^[1-9]\d* minutes?$") Then
		$coef = 1 ; 1 minute
	Else
		Return 0 ; resultat non renseign ou incorrect
	EndIf
	Return $coef * Int(StringLeft($timeOut, StringInStr($timeOut," ")))
EndFunc

Func convertIntToTimeOut($int)
	If     $int >= 535680 And $int < 5356800 And Mod($int,535680) = 0 Then ; entre 1 et 10 ans (5356800 minutes)
		Local $valeur = $int/535680
		Return $valeur&" an"&cond($valeur>1,"s","")
	ElseIf $int >= 44640  And $int < 535680   And Mod($int,44640) = 0  Then ; entre 1 et 12 mois = 1 an (535680 minutes)
		Local $valeur = $int/44640
		Return $valeur&" mois"
	ElseIf $int >= 1440   And $int < 44640    And Mod($int,1440) = 0   Then ; entre 1 et 31 jours = 1 mois (44640 minutes)
		Local $valeur = $int/1440
		Return $valeur&" jour"&cond($valeur>1,"s","")
	ElseIf $int >= 60     And $int < 1440     And Mod($int,60) = 0     Then ; entre 1 et 24 heures = 1 jour (1440 minutes)
		Local $valeur = $int/60
		Return $valeur&" heure"&cond($valeur>1,"s","")
	ElseIf $int >= 1      And $int < 60       And Mod($int,1) = 0      Then ; entre 1 et 60 minutes = 1 heure
		Local $valeur = $int/1
		Return $valeur&" minute"&cond($valeur>1,"s","")
	EndIf
	Return "Aucun" ; resultat non renseign ou incorrect
EndFunc