#Include-Once

; Librairies nAiO
#Include "..\Encryptage Decryptage.au3"

Global Const $DEFAULT_PROFIL = "Profil"

#Region > Modlisation des objets "Profil"

; ~~~ Prsentation de la structure ~~~

; L'objet "Profil" est une reprsentation d'un dossier de profil du nAiO; lui sont rattachs les attributs "privs" suivants :
;  - nom    [string-64]  : nom court du dossier li au profil
;  - chemin [string-256] : chemin complet du dossier du profil
; La liste de profils est une reprsentation des profils disponibles; lui est rattache la modlisation suivante :
;  - Array de taille n+1, o n reprsente le nombre de profils disponibles
;    - [indice 0] : valeur gale au nombre de profils disponibles, soit n
;    - [indice 1] : premier profil (objet "Profil")
;    - ...
;    - [indice n] : (n)ime profil (objet "Profil")
;  - Une liste vierge de profils est donc reprsente ainsi : Dim $array[1] = [0]



; ~~~ Constructeurs et fonctions de base ~~~

; ### newProfil($nom,$chemin)>[Profil] :
; Construit un objet "Profil"  partir des donnes $nom et $chemin, et retourne sa valeur
; => $valeur est optionnel
Func newProfil($nom="", $chemin="")
	Local $profil = DllStructCreate(getStructureProfil())
	If $nom = "" Then Return ""
	setProfilNom($profil, $nom)
	setProfilChemin($profil, $chemin)
	Return $profil
EndFunc
; ### copyProfil($oldProfil)>[Profil] :
; Construit un objet "Profil"  partir des donnes dupliques d'un autre objet "Profil", et retourne la nouvelle valeur
Func copyProfil($oldProfil)
	Local $newProfil = newProfil(), $structure = getStructureProfilArray()
	For $i = 0 To getStructureProfilNumber()-1
		copyProfilAttribute($newProfil, $oldProfil, $structure[$i][1])
	Next
	Return $newProfil
EndFunc
; ### deleteProfil($profil)>void :
; Detruit l'objet "Profil" $profil
Func deleteProfil(ByRef $profil)
	$profil = 0
EndFunc
; ### isProfil($profil)>boolean :
; Retourne la valeur True si $profil est bien un objet "Profil", False sinon
Func isProfil($profil)
	Local $i = 0, $structure = getStructureProfilArray()
	If Not IsDllStruct($profil) Then
		Return False
	EndIf
	For $i=0 To getStructureProfilNumber()-1
		Local $valeur = DllStructGetData($profil, $structure[$i][1])
		If @error Then Return False
	Next
	Return True
EndFunc
; ### toStringProfil($profil)>string :
; Retourne une chaine de caractres identifiant un objet "Profil"
Func toStringProfil($profil)
	Return getProfilNom($profil)
EndFunc
; ### newListeProfils()>array :
; Cre une liste vierge de profils
Func newListeProfils()
	Local $retour[1]
	setListeProfilsLength($retour,0)
	Return $retour
EndFunc
; ### deleteListeProfils($liste)>void :
; Ecrase $liste avec une liste vierge d'objets "Profil" aprs avoir dtruit chaque objet "Profil" prsent
Func deleteListeProfils(ByRef $liste)
	If IsArray($liste) Then
		For $i = 0 To UBound($liste)-1
			$liste[$i] = 0
		Next
	EndIf
	$liste = newListeProfils()
EndFunc



; ~~~ Fonctions de dbugage ~~~

; ### displayProfil($profil)>void :
; Affiche les informations de $profil dans une fentre type MsgBox
Func displayProfils($liste)
	For $i = 1 To getListeProfilsLength($liste)
		displayProfil(getProfilInListAt($liste,$i),$i&"/"&getListeProfilsLength($liste))
	Next
EndFunc
; ### displayProfils($liste)>void :
; Afficher les informations des lments de $liste dans des fentres ordonnes type MsgBox
Func displayProfil($profil, $title="")
	Local $fullTitle = "Profil"
	If $title<>"" Then $fullTitle = $fullTitle & " ["&$title&"]"
	If isProfil($profil) Then
		Local $structure = getStructureProfilArray(), $msg = _
			"Profil.toString: '" & toStringProfil($profil) & "'" & @CRLF & _
			"Struct Size: " & DllStructGetSize($profil) & @CRLF & _
			"Struct pointer: " & DllStructGetPtr($profil) & @CRLF & _
			"Data:"
		For $i = 0 To getStructureProfilNumber()-1
			$msg &= @CRLF & @TAB & $structure[$i][1]&": '"&getProfilAttribute($profil, $structure[$i][1])&"'"
		Next
		MsgBox(0, $fullTitle, "{"&@CRLF&$msg&@CRLF&"}" )
	Else
		MsgBox(16, $fullTitle, "!!! Ce profil n'est pas un objet conforme !!!" )
	EndIf
EndFunc



; ~~~ Fonctions Get/Set pour les attributs index, alias et avatar ~~~

; ### getProfilNom($profil)>string :
; Lit l'attribut nom de $profil, et retourne sa valeur
Func getProfilNom($profil)
	Return getProfilAttribute($profil, "nom")
EndFunc
; ### setProfilNom($profil,$nom)>void :
; Ecrit la valeur $nom dans l'attribut nom de $profil
Func setProfilNom(ByRef $profil, $nom)
	setProfilAttribute($profil, "nom", $nom)
EndFunc
; ### getProfilChemin($profil)>string :
; Lit l'attribut chemin de $profil, et retourne sa valeur
Func getProfilChemin($profil)
	Return getProfilAttribute($profil, "chemin")
EndFunc
; ### setProfilChemin($profil,$chemin)>void :
; Ecrit la valeur $chemin dans l'attribut chemin de $profil
Func setProfilChemin(ByRef $profil, $chemin)
	setProfilAttribute($profil, "chemin", $chemin)
EndFunc
; ### getProfilDate($profil)>string :
; Retourne la date lie  l'objet "Profil" $profil
Func getProfilDate($profil)
	Local $date = FileGetTime(StringTrimRight(getProfilChemin($profil),1),1,1)
	If $date = 0 Then SetError(1,0,@YEAR&@MON&@MDAY&@HOUR&@MIN&@SEC)
	Return $date
EndFunc
; ### setProfilDate($profil,$date)>void :
; Enregistre la valeur $date sur la date lie  l'objet "Profil" $profil
Func setProfilDate($profil, $date)
	FileSetTime(StringTrimRight(getProfilChemin($profil),1), $date,1)
EndFunc



; ~~~ Fonctions de listing de profils ~~~

; ### addToListeProfils($liste, $profil)>void :
; Ajoute l'objet "Profil" $profil  la fin de la liste de profils $liste
; => Si un objet "Profil" ayant la mme valeur pour l'attribut nom existe dj dans la liste, aucune action n'est effectue
Func addToListeProfils(ByRef $liste, $profil)
	Local $nom = getProfilNom($profil)
	If Not isProfil(getProfilInListFromNom($liste,$nom)) Then
		setListeProfilsLength($liste,getListeProfilsLength($liste)+1)
		setProfilInListAt($liste,getListeProfilsLength($liste),$profil)
	EndIf
EndFunc
; ### getProfilCheminInListFromNom($liste,$nom)>string :
; Retourne l'attribut chemin de l'objet "Profil" prsent dans la liste de profils $liste ayant son attribut nom gal  $nom
; => Si aucun objet "Profil" ne rpond au critre, la chaine vide "" est retourne
Func getProfilCheminInListFromNom($liste,$nom)
	Return getProfilChemin(getProfilInListFromNom($liste,$nom))
EndFunc
; ### getProfilInListFromNom($liste,$nom)>[Profil] :
; Retourne l'objet "Profil" prsent dans la liste de profils $liste ayant son attribut nom gal  $nom
; => Si aucun objet "Profil" ne rpond au critre, la chaine vide "" est retourne
Func getProfilInListFromNom($liste,$nom)
	For $i = 1 To getListeProfilsLength($liste)
		If isProfil(getProfilInListAt($liste,$i)) And getProfilNom(getProfilInListAt($liste,$i))=$nom Then Return getProfilInListAt($liste,$i)
	Next
	Return ""
EndFunc
; ### setProfilCheminInListFromNom($liste,$nom,$chemin)>[Profil] :
; Modifie l'objet "Profil" prsent dans la liste de profils $liste ayant son attribut nom gal  $nom, en crivant $chemin dans son attribut chemin, et retourne l'lment avant modification
; => Si aucun objet "Profil" ne rpond au critre, la chaine vide "" est retourne
Func setProfilCheminInListFromNom(ByRef $liste,$nom,$chemin)
	For $i = 1 To getListeProfilsLength($liste)
		Local $modified = getProfilInListAt($liste,$i)
		If isProfil($modified) And getProfilNom($modified)=$nom Then
			setProfilInListAt($liste,$i,newProfil($nom,$chemin))
			Return $modified
		EndIf
	Next
	Return ""
EndFunc
; ### getProfilInListAt($liste,$i)>[Profil] :
; Retourne l'objet "Profil" prsent  l'indice $i dans la liste de profils $liste
; => Si l'indice $i ne contient pas un objet "Profil", la chaine vide "" est retourne
Func getProfilInListAt($liste,$i)
	If isProfil($liste[$i]) Then
		Return $liste[$i]
	Else
		Return ""
	EndIf
EndFunc
; ### setProfilInListAt($liste,$i,$profil)>void :
; Ecrit l'objet "Profil" $profil  l'indice $i de la liste de profils $liste
; => Si l'indice $i est invalide, aucune action n'est effectue
Func setProfilInListAt(ByRef $liste,$i,$profil)
	If $i>0 And $i<=getListeProfilsLength($liste) Then
		$liste[$i] = $profil
	EndIf
EndFunc
; ### getListeProfilsLength($liste)>int :
; Retourne le nombre d'objets "Profil" prsent dans la liste de profils $liste
Func getListeProfilsLength($liste)
	Return $liste[0]
EndFunc
; ### setListeProfilsLength($liste,$length)>void :
; Redimensionne  $length la taille de la liste de profils $liste
Func setListeProfilsLength(ByRef $liste, $length)
	If $length>=0 Then
		$liste[0] = $length
		ReDim $liste[getListeProfilsLength($liste)+1]
	EndIf
EndFunc



; ~~~ Fonctions pour mashall/unmarshall ~~~

; ### convertStructureToListeProfils($liste)>void :
; Lit les dossiers de profils dans $nAiO_repertoire_data, et enregistre la liste de profils correspondante dans $liste
; => Les donnes de $liste sont crases
; => Le profil par dfaut est mis  jour
Func convertStructureToListeProfils(ByRef $liste)
	$liste = newListeProfils()
	Local $profils = _FileListToArray($nAiO_repertoire_data,"*",2)
	If Not isArray($profils) Then
		DirCreate($nAiO_repertoire_data&$DEFAULT_PROFIL)
		$profils = _FileListToArray($nAiO_repertoire_data,"*",2)
	EndIf
	If isArray($profils) Then
		For $i = 1 To $profils[0]
			Local $path = $nAiO_repertoire_data&$profils[$i]&"\"
			Local $profil = newProfil($profils[$i],$path)
			FileClose(FileOpen(getProfilFichier("config",     $profil), 9))
			FileClose(FileOpen(getProfilFichier("comptes",    $profil), 9))
			FileClose(FileOpen(getProfilFichier("multis",     $profil), 9))
			FileClose(FileOpen(getProfilFichier("alertes",    $profil), 9))
			FileClose(FileOpen(getProfilFichier("modules",    $profil), 9))
			FileClose(FileOpen(getProfilFichier("liens",      $profil), 9))
			FileClose(FileOpen(getProfilFichier("historique", $profil), 9))
			If @error = 0 Then addToListeProfils($liste, $profil)
		Next
	EndIf
	setProfilActuel(getLastUsedProfil($liste))
EndFunc



; ~~~ Fonctions internes  la modlisation,  ne pas utiliser hors de ce fichier ~~~

; ### getStructureProfilArray()>matrix :
; Retourne une matrice contenant les attributs et leurs formats, pour structurer un objet "Profil"
Func getStructureProfilArray()
	Local $structure[2][3] = [["char","nom",64], ["char","chemin",256]]
	Return $structure
EndFunc
; ### getStructureProfilNumber()>int :
; Retourne le nombre d'attributs ncessaires pour structurer un objet "Profil"
Func getStructureProfilNumber()
	Return UBound(getStructureProfilArray(),1)
EndFunc
; ### getStructureProfil()>string :
; Retourne une chaine de caractres contenant les attributs et leurs formats, pour structurer un objet "Profil"
Func getStructureProfil()
	Local $i = 0, $string = "", $structure = getStructureProfilArray()
	For $i=0 To getStructureProfilNumber()-1
		If $string <> "" Then $string &= ";"
		$string &= $structure[$i][0] & " " & $structure[$i][1] & cond($structure[$i][2]<>"", "["&$structure[$i][2]&"]", "")
	Next
	Return $string
EndFunc
; ### getProfilAttribute($profil,$attribut)>string :
; Retourne l'attribut $attribut de l'objet "Profil" $profil
Func getProfilAttribute($profil, $attribut)
	Return DllStructGetData($profil, $attribut)
EndFunc
; ### setProfilAttribute($profil,$attribut,$chemin)>void :
; Ecrit la valeur $chemin dans l'attribut $attribut de l'objet "Profil" $profil
Func setProfilAttribute(ByRef $profil, $attribut, $chemin)
	DllStructSetData($profil, $attribut, $chemin)
EndFunc
; ### copyProfilAttribute($newProfil,$oldProfil,$attribut)>void :
; Ecrit la valeur de l'attribut $attribut de l'objet "Profil" $oldProfil dans celui de l'objet "Profil" $newProfil
Func copyProfilAttribute(ByRef $newProfil, $oldProfil, $attribut)
	setProfilAttribute($newProfil, $attribut, getProfilAttribute($oldProfil, $attribut))
EndFunc

#EndRegion

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

#Region > Modlisation du fonctionnement du profil en cours
Func getProfilFichier($alias="", $profil="")
	Local $profilChemin = getProfilChemin(getProfilActuel())
	If isProfil($profil) Then $profilChemin = getProfilChemin($profil)
	Switch StringLower($alias)
	Case "config"
		Return $profilChemin&"Config.ini"
	Case "comptes"
		Return $profilChemin&"Comptes.data"
	Case "multis"
		Return $profilChemin&"Multis.data"
	Case "alertes"
		Return $profilChemin&"Alertes.data"
	Case "modules"
		Return $profilChemin&"Modules.data"
	Case "liens"
		Return $profilChemin&"Liens.data"
	Case "historique"
		Return $profilChemin&"Historique.log"
	EndSwitch
	SetError(1,0,"")
EndFunc
Func checkProfilPassword($clef, $profil="")
	Local _
		$hash_to_compare = autoencryptage($clef), _
		$hash_to_read    = IniRead(getProfilFichier("config",$profil),"ZaDS-nAiO","Password","")
	Return ($hash_to_compare = $hash_to_read)
EndFunc
Func isProfilProtected($profil="")
	Local $hash = IniRead(getProfilFichier("config",$profil),"ZaDS-nAiO","Password","")
	Return $hash<>"" And $hash<>autoencryptage("")
EndFunc
Func getProfilActuel()
	Return $nAiO_repertoire_data_profil
EndFunc
Func setProfilActuel($profil)
	$nAiO_repertoire_data_profil = $profil
EndFunc
Func getLastUsedProfil($listeProfils)
	Local $t = 0, $resultat
	For $i = 1 To getListeProfilsLength($listeProfils)
		Local $profil = getProfilInListAt($listeProfils,$i)
		Local $t_loop = Number(FileGetTime(StringTrimRight(getProfilChemin($profil),1),1,1))
		If $t_loop > $t Then
			$resultat = $profil
			$t = $t_loop
		EndIf
	Next
	If $t = 0 Then
		SetError(1,0,"")
	Else
		Return $resultat
	EndIf
EndFunc
Func setLastUsedProfil()
	setProfilDate($nAiO_repertoire_data_profil, @YEAR&@MON&@MDAY&@HOUR&@MIN&@SEC)
EndFunc
#EndRegion